#compare stages from Cabecera (Pardinella and weir) (to be run after harmonize_stages.R)

#settings
gauges2do="Cabecera"
stage_file_dir="./hdata/corr/complementary/" #directory containing complementary stage records

  
#load logger data  
  source("f_read_loggerdata.R")
  gauge="Cabecera"; datum="weir"; device="TT"
  logger_data_weir=read_loggerdata(gauge, datum, device, stage_file_dir="./hdata/corr/", do_correct_tz = FALSE) #try to read logger data
  summary(logger_data_weir)  

  gauge="Cabecera"; datum="bridge"; device="TT"
  logger_data_bridge=read_loggerdata(gauge, datum, device, stage_file_dir=stage_file_dir) #try to read logger data
  summary(logger_data_bridge)  

  timestep_representation=0.5   #how are records assigned to output resolution. Example for 1 h resolution  
  #0: all data between 11:00 and 12:00 is assigned to record with timestamp 12:00
  #0.5: all data between 11:30 and 12:30 is assigned to record with timestamp 12:00
  #1: all data between 12:00 and 13:00 is assigned to record with timestamp 12:00                                
  time_step=as.difftime(5, units="mins")
  aggr_fun=mean
  somedate=ISOdate(2000,1,1)  #aux for date operations
  origin=somedate-as.numeric(somedate)
  time_interval_minutes=as.numeric(time_step)

  start_date_vec=min(min(logger_data_weir$date),min(logger_data_bridge$date))
  end_date_vec  =max(max(logger_data_weir$date),max(logger_data_bridge$date))

  
  time_line = seq(from=start_date_vec-2*time_step, to=end_date_vec+2*time_step, by=time_step, tz = "GMT")  #generate time sequence
  time_line = floor(as.numeric(time_line)/60/time_interval_minutes) * time_interval_minutes * 60
  time_line = as.POSIXct(time_line, origin=origin,tz="GMT")

for (i in 1:2)
{
  if (i==1)
    logger_data=logger_data_weir  else
    logger_data=logger_data_bridge

  resampled_data=rep(0*NA,length(time_line))     

  logger_data$stage[which(is.nan(logger_data$stage))]=NA   #replace NaN with NA

  intermittent=FALSE       #regular sampling interval

  time_rounded = floor(as.numeric(logger_data$date)/60/time_interval_minutes + timestep_representation) * time_interval_minutes * 60
  time_rounded = as.POSIXct(time_rounded, origin=origin)

  aggr=aggregate(logger_data$stage, by=list(date=time_rounded), FUN=mean)

  resamp=merge(data.frame(date=time_line), aggr, by="date", all.x=TRUE)
  if (i==1)
  {
    names(resamp)[2]="stage_weir"
    resampled=resamp  
  }  else
    resampled$stage_bridge=resamp[,2]
  
}


startdate=ISOdatetime(2011,10,26, 23, 59, 0, tz="CET") #Jose's objection at Cabecera: log jam
enddate  =ISOdate(2011,10,28, 0, 0, tz="CET")

#both datasets against each other and find faulty period
  subs= (resampled$date >= startdate & resampled$date<=enddate)
  plot(resampled$stage_weir, resampled$stage_bridge)
  points(resampled$stage_weir[subs], resampled$stage_bridge[subs], col="red")
  
#remove more faulty data - repeat this block until satisfied  
{
  windows()
  plot(resampled$stage_weir[!subs], resampled$stage_bridge[!subs])
  a=identify(resampled$stage_weir[!subs], resampled$stage_bridge[!subs],labels=resampled$date[!subs])
  sort(resampled$date[!subs][a])

  subs = subs | (resampled$date %in% resampled$date[!subs][a])
  sum(subs)
}
  format(resampled$date[subs], tz="CET")
  #blocks removed
#   2011-07-13 09:40:00 - 2011-07-13 10:40:00
#   2011-08-25 10:20:00 - 2011-08-25 10:25:00
#   2011-10-25 05:45:00 - 2011-10-25 06:45:00
#   2011-10-27 00:00:00 - 2011-10-27 23:55:00
#   
  
#   startdate=ISOdate(2011,10,20) #Jose's objection at Cabecera
#   enddate  =ISOdate(2011,11,1)
#   subs2= subs & !(resampled$date >= startdate & resampled$date<=enddate)
#   points(resampled$stage_weir[subs2], resampled$stage_bridge[subs2], col="blue")

  
  
  resampled$stage_weir[subs]=NA
  summary(resampled)

# establish relation between both TT-gauges
  stage_plot=seq(from=min(resampled$stage_bridge, na.rm=TRUE), to=max(resampled$stage_bridge, na.rm=TRUE), length.out=40)
  find_closest=function(series, target) #find closest match of target in series
  {return(which.min(abs(series-target))[1])}

  pairwise_complete_i = is.finite(resampled$stage_bridge) & is.finite(resampled$stage_weir)
  regression_i = sapply(X=stage_plot, FUN=find_closest, series=resampled$stage_bridge[pairwise_complete_i]) #subset to feed into regression (ensures equalt point density over entire range)

  
  conv_mod = lm(formula=stage_weir~I(stage_bridge^2) + stage_bridge, data=resampled[pairwise_complete_i,][regression_i,])
  
  
#plot relation
  bb=predict(conv_mod, newdata=data.frame(stage_bridge=stage_plot))
  
  points(resampled$stage_weir[pairwise_complete_i][regression_i], resampled$stage_bridge[pairwise_complete_i][regression_i], col="red")
  lines(bb, stage_plot, col="green")

#convert resampled bridge data to weir and fill holes in weir data
  resampled$stage_weir2=predict(conv_mod, newdata=resampled) 

  org_weir= !is.na(resampled$stage_weir) #find timesteps to fill with original weir data
  resampled$stage_weir2[org_weir]=resampled$stage_weir[org_weir]

  plot(resampled$stage_weir2, resampled$stage_bridge)


#convert original bridge data to weir and fill holes in weir data
  slice_bridge = (logger_data_bridge$date >= startdate & logger_data_bridge$date<=enddate)
  converted_stage=predict(conv_mod, newdata=data.frame(stage_bridge=logger_data_bridge$stage[slice_bridge])) #convert stage of selected time slice
  
  slice_weir = (logger_data_weir$date >= startdate & logger_data_weir$date<=enddate)
  logger_data_weir$date [slice_weir] = logger_data_bridge$date [slice_bridge]
  logger_data_weir$stage[slice_weir] = converted_stage

  # logger_data_weir =  rbind(
  #         logger_data_weir[logger_data_weir$date < startdate ,],
  #         data.frame(date=logger_data_bridge$date [slice_bridge], stage=converted_stage)
  #         logger_data_weir[logger_data_weir$date > enddate   ,]
  #         
  
  file.rename(from="hdata/corr/Cabecera_TT_weir_corr.txt", to = "hdata/corr/Cxabecera_TT_weir_corr_unfilled.txt")
  write.table(file="hdata/corr/Cabecera_TT_weir_corr.txt", 
            data.frame(date=format(logger_data_weir$date, "%Y-%m-%d %H:%M", tz="etc/GMT-1"),
                  stage=logger_data_weir$stage*1000), sep="\t", quote=FALSE, row.names=FALSE) #write stage data
            


