
do_h0_correction=TRUE #correct for changing streambed elevation (useful for Carrasquero, Villacarli and Ceguera)

if (do_h0_correction)
#correct h0 for Villacarli
{
   gauge="Villacarli"
   device="RQ24"
   datum="downstream-left"
#   
  # gauge="Ceguera"
  # device="TT"
  # datum="bridge"
  
  # gauge="Carrasquero"
  # device="TT"
  # datum="bridge"  
  
  
  stage_file_dir="./hdata/" #directory containing stage records  
  source("f_read_loggerdata.R") 
  
logger_data=na.omit(read_loggerdata(gauge, datum, device, stage_file_dir="hdata/corr/", do_correct_tz = FALSE)) #try to read logger data
if (is.null(logger_data)) stop("corrected file not found for current gauge and datum")
if (max(logger_data$stage)< 0.1) logger_data$stage = logger_data$stage * 1000 #convert faulty units
#summary(logger_data)




lo_flo_file=sub(paste(stage_file_dir,"corr/",gauge,"_",device,"_",datum,"_loflophases.RData",sep=""), pattern = "\\.txt$", repl=".h0")

if (file.exists(lo_flo_file))
{
  load(lo_flo_file) 
  if (any(lf_phase$date_begin != logger_data$date[lf_phase$begin])) stop("inconsistency of logger file and lo-flo correction. Fix manually or delete loflo correction file.")
} else  
{
  lf_phase=data.frame(begin = NA,
                      prior_peak=NA,
                      trans_end=NA, 
                      marked=NA,
                      end=NA,
                      level=NA)[-1,]
  }


#identify base-flow periods interactively
plot_phases = function(x, y, lf_phase, xlim=NULL, ...)  
{
  if (is.null(xlim)) xlim=range(x, na.rm=TRUE)
  
  subset_spacing=12
  subset4display=seq(from=1, to=length(x), by=subset_spacing) #subset for display, otherwise, the display often crashes
  
  plot(x[subset4display], y[subset4display], main="pick flood peaks causing changes in h0-level or lo-flo-levels,\n export via Labels -> save IDs (MyIds or MyIds_lo)", 
       type="l", xlab="date", ylab="stage", xlim=xlim, xaxt="n", ...) 
  
  #xticks need to be handled extra; otherwise, they are not always plotted
  if ("POSIXt" %in% attr(x,"class") ) #if x-data is a time
  {
    org_tz=attr(x,"tzone")  #get timezone of original x-data
    if (is.numeric(xlim[1]))  #sometimes during zooming, xlim is used in numeric form instead of POSIX-format
      xlim =  as.POSIXct(xlim, origin="1970-01-01 00:00.00 UTC")
    xlim = as.POSIXct(as.POSIXlt(xlim, tz=org_tz))
  }
  ticks=pretty( xlim)
  a=axis(side=1, at=ticks, labels=ticks)
  
  if (nrow(lf_phase)>0)
  for (i in 1:nrow(lf_phase))
  {
    #lo flow phase
    lines(x[c(lf_phase$trans_end[i], lf_phase$end[i])],
          rep(lf_phase$level[i],2), col=i+1)
    #manual detection point
    points(x[lf_phase$marked[i]]    , lf_phase$level[i], col=i+1, pch="X")
    points(x[lf_phase$marked[i]]    , y[lf_phase$marked[i]], col=i+1, pch="X")
    #responsible peak
    points(x[lf_phase$prior_peak[i]]    , lf_phase$level[i], col=i+1, pch="^")
    points(x[lf_phase$prior_peak[i]]    , y[lf_phase$prior_peak[i]], col=i+1, pch="^")
    
    #transition
    if (i>1)
    {
      lines(x[c(lf_phase$begin[i], lf_phase$trans_end[i])],
            lf_phase$level[c(i-1,i)], col=i+1)
      points(x[lf_phase$begin[i]]    , lf_phase$level[i-1], col=i+1, pch="*")
      points(x[lf_phase$begin[i]]    , y[lf_phase$begin[i]], col=i+1, pch="*")
      points(x[lf_phase$trans_end[i]]    , lf_phase$level[i], col=i+1, pch="*")
      points(x[lf_phase$trans_end[i]]    , y[lf_phase$trans_end[i]], col=i+1, pch="*")
    }
  }
  legend("topright", legend = c("determined low-flow level", "responsible peak", "transition"),
         pch=c("X","^","*"), bg = "white") 
}  

#construct low-flow phases and transitions
derive_lo_flow_breakpoints <- function (lf_phase, logger_data) 
{
 if (nrow(lf_phase)==0) return(lf_phase)
  #overlap = lf_phase$end[-nrow(lf_phase)] > lf_phase$begin[-1]
  # overlap = (lf_phase$end   < lf_phase$prior_peak) | (lf_phase$begin > lf_phase$prior_peak)
  # if (any(overlap)) #may happen with peaks added in a later iteration
  # {
  #   
  # }    
  lf_phase[, c("begin", "trans_end", "end", "level") ]  =NA
  
  repeatrun=0
  for (i in 1:nrow(lf_phase))
  {
    if (is.na(lf_phase$marked[i]))
      lf_phase$marked[i] = lf_phase$prior_peak[i]-1 +
        which.min(logger_data$stage[lf_phase$prior_peak[i] : 
                                      min(lf_phase$prior_peak[i+1], nrow(logger_data), na.rm=TRUE)])
    
    if (i==1) 
    {  
      lf_phase$begin    [i] = 1
      lf_phase$trans_end[i] = 1 
      lf_phase$end      [i] = lf_phase$marked[i] +1
      if (!is.na(lf_phase$marked[i]))
      lf_phase$level    [i] = logger_data$stage[lf_phase$marked[i]] else
      lf_phase$level    [i] = min(logger_data$stage[1 : lf_phase$marked[i]])
    }  else
    {
      #find peak (if not already set)
      if (is.na(lf_phase$prior_peak[i]))
      {
        peak_before = which.max(logger_data$stage[lf_phase$marked[i-1] : lf_phase$marked[i]])
        lf_phase$prior_peak[i]    = lf_phase$marked[i-1] + peak_before -1
      }
      
      #find transition start
      range_before = diff(range(logger_data$stage[lf_phase$marked[i-1] : lf_phase$prior_peak[i]]), na.rm=TRUE)
      begin = max(which(logger_data$stage[lf_phase$marked[i-1] : lf_phase$prior_peak[i]] <
                          logger_data$stage[lf_phase$prior_peak[i]] - 0.1 * range_before))
      lf_phase$begin[i]    = lf_phase$marked[i-1] + begin
      lf_phase$end  [i-1]  = lf_phase$begin[i] - 1
      
      #find transition end
      range_after = diff(range(logger_data$stage[lf_phase$prior_peak[i]: min(lf_phase$marked[i], lf_phase$prior_peak[i+1], na.rm=TRUE) ]), na.rm=TRUE)
      end = min(which(logger_data$stage[lf_phase$prior_peak[i]: lf_phase$marked[i]] <
                        (logger_data$stage[lf_phase$prior_peak[i]] - 0.1 * range_after)))
      lf_phase$trans_end[i]    = lf_phase$prior_peak[i] + end
      
      #find level
      lf_phase$level[i] = min(logger_data$stage[lf_phase$trans_end[i] : lf_phase$marked[i]])
      # lf_phase$level[i] = quantile(logger_data$stage[lf_phase$trans_end[i] : lf_phase$marked[i]], probs = 0.05) -
      #    diff(quantile(logger_data$stage[lf_phase$trans_end[i] : lf_phase$marked[i]], probs = c(0.25,0.75)))
      
      if (i == nrow(lf_phase)) #last phase
        lf_phase$end  [i]  = length(logger_data$stage)
      
      if (repeatrun!=0) #should a period be treated again, now that the following is set?
        i=repeatrun
      
    }
    
  } 
  return(lf_phase)
} 

  library(playwith)
  #pick low-flow points with "Identify", save to variable via "Labels, Save ID"
  myIds=myIds_lo=NULL
  
  ###repeat this until satisfied
  
  
  #stop("resume script from here manually")
 
  while(TRUE) 
  {
    for (i in myIds)
    {
      if (i %in% lf_phase$prior_peak) next #already contained
      lf_phase = rbind(lf_phase, lf_phase[nrow(lf_phase)+1,]) #add empty row
      lf_phase$prior_peak[nrow(lf_phase)]=i #add manually detected peak
    }
    lf_phase =  lf_phase[sort.int(lf_phase$prior_peak, index.return = TRUE)$ix,]
    
    for (i in myIds_lo)
    {
      if (i %in% lf_phase$marked) next #already contained
      corresponding_phase = which(lf_phase$begin < i & lf_phase$end > i) #find the phase the marker belongs to  
      lf_phase$marked[corresponding_phase] = i #add manually detected lo flow level
      lf_phase$level = NA #mark this phase as to be re-processed
    }
    lf_phase =  lf_phase[sort.int(lf_phase$prior_peak, index.return = TRUE)$ix,]
    
  
    lf_phase = derive_lo_flow_breakpoints(lf_phase, logger_data) #create phases, first shot
   
  
    #verify results, refine
    myIds=myIds_lo=NULL
    playwith(new=TRUE, labels = 1:length(logger_data$date),
             plot_phases(x=logger_data$date,  y=logger_data$stage, lf_phase = lf_phase)
    )
    commnt=readline("press ENTER or pick flood peaks, export via Label > SaveIDs, finish with ENTER ")
    if (is.null(myIds)) break
  }


  lf_phase$date_begin = logger_data$date[lf_phase$begin] #just for later consistency checks
  save(file=paste(stage_file_dir,"corr/",gauge,"_",device,"_",datum,"_loflophases.RData",sep=""),list="lf_phase")

  windows()
  plot_phases(x = logger_data$date, y = logger_data$stage, lf_phase = lf_phase)
  savePlot(file=sub(x=lo_flo_file, pattern = "RData", repl="wmf"))
  
  
#rolling quantiles (long computation, illustrates breaks in h0, not too useful)
#   windowsizes= c(30,60,120,240)*24*60 /subset_spacing / as.numeric(median(diff(logger_data$date[1:100], unit="min")))
#   
#   library(zoo)
#   
#   roll_quant=array()
#   for (i in 1:length(windowsizes)){
#     rge = rollapply(data = logger_data$stage[subset4display], width=windowsizes[i], fill=NA, FUN=quantile, prob=0.05)
#     roll_quant=rbind(roll_quant,rge)
#   }
#   
#   library(zoom)
#   windows()
#   plot(logger_data$date[subset4display], logger_data$stage[subset4display], main="recalibrated and merged: pick points for construction zero-flow-level", type="l") #, pch=".")
#   for (i in 1:nrow(roll_quant))
#     lines(logger_data$date[subset4display[1:ncol(roll_quant)]],
#           roll_quant[i,], col=i)
#   zm()    



#display velocity data (test) - could help veryifying determined breaks based on velocity
#   rq24=read.table(file = "hdata/test_v/t.txt", header = FALSE, sep="\t", na.strings = "-7999")
#   plot(rq24$V3, type="l", ylim=c(0,3))
#   lines(rq24$V4, col="red")
#   zm()    
#   identify(1:length(rq24$V3), rq24$V3, labels = 1:length(rq24$V3))


#lo_flow = rge <= max_range

#construct correction 
  #linear interpolation including transition phases
  approx_data=data.frame(x=logger_data$date[sort(c(lf_phase$trans_end, lf_phase$marked, lf_phase$end))],y=rep(lf_phase$level, each=3))
  
  lo_flo_fun = approxfun (approx_data , method="linear", rule=2)
  
  lo_flo =lo_flo_fun(logger_data$date)
  
  windows()
  plot(logger_data$date, logger_data$stage-lo_flo, main="stage, h0_corrected", type="l", ylim=c(-1,2))
  lines(logger_data$date, lo_flo, col="red")
  #savePlot(file=sub(x=lo_flo_file, pattern = "RData", repl="wmf"))
  
  #lo_flo_fun=approxfun(x=logger_data$date[lo_flo_points],y=logger_data$stage[lo_flo_points], method="linear", rule=2)
  #save(file=paste(stage_file_dir,"corr/",gauge,"_",device,"_",datum,"_loflo.RData",sep=""),list="lo_flo_fun")
  #write h0 corrected time series
  #write.table(file=paste(stage_file_dir,"corr/",gauge,"_",device,"_h0_corr.txt",sep=""), 
  #            data.frame(date=format(logger_data$date, "%Y-%m-%d %H:%M", tz="etc/GMT-1"), stage=logger_data$stage-lo_flo$y),
  #            sep="\t", quote=FALSE, row.names=FALSE) 

  #write h0 break points
  write.table(file=sub(x=lo_flo_file, pattern = "loflophases.RData", repl="corr.h0"), 
              data.frame(date=format(approx_data$x, "%Y-%m-%d %H:%M", tz="etc/GMT-1"), stage=approx_data$y),
              sep="\t", quote=FALSE, row.names=FALSE) 

  save(file=paste(stage_file_dir,"corr/",gauge,"_",device,"_",datum,"_loflophases.RData",sep=""),list=c("lf_phase", "lo_flo_fun"))
  
  

# #complement file with q-measurements
#   meas2 = meas[meas$gauge==gauge & !is.na(meas$discharge_.m..s.),]
#   find_closest=function(series, target) #find closest match of target in series
#   {return(which.min(abs(series-target))[1])}
#   meas_times=sapply(meas2$date, FUN = find_closest, series=logger_data$date) #try to find corresponding time
#   
#   meas2$stage=logger_data$stage[meas_times]-lo_flo$y[meas_times] #replace original stage values with h0-corrected ones
#   
#   time_diffs = difftime(meas2$date, logger_data$date[meas_times], units="min")
#   
#   meas2 = meas2[abs(time_diffs)<=3,] #discard records with no match within 3 minutes
#   #summary(logger_data)
#   #tail(meas2)
#   meas2$stage_datum="h0corr"
#   
#   
#   file.copy(meas_file, paste0(meas_file,"_bak"))
#   write.table(file=meas_file, meas2, quote=FALSE, row.names=FALSE, sep="\t", append=TRUE, col.names = FALSE)
#   plot(meas2$stage, meas2$discharge_.m..s.)

}
