#convert between different gauge datum's based on information in data frame "convert_info"
#"convert_info" can be read from file "gauge_datum_conv.txt"

prepare_convert_info = function()
{  
#reformat convert_info file and prepare gauge_datum_conv.txt (only to be done once)
  convert_file="../wq/gauge_datum.txt" #to be saved as txt from P:\sesam_data\SESAMII\Data_Fieldwork\Spain\Discharge_Gauges\Discharge_metering\gauge_datum.xlsx
  convert_info  = read.table(file=convert_file, header=TRUE, sep="\t", stringsAsFactors=FALSE)
  convert_info  = convert_info[,c("gauge","datum","target_datum", "conversion2target_datum")]
  
  convert_info = merge(convert_info, convert_info[convert_info$target_datum==1,c("gauge","datum")], by="gauge")
  convert_info$remarks=""
  convert_info$remarks[convert_info$target_datum==1]="standard"
  convert_info$target_datum=NULL
  names(convert_info)[c(2,4)]=c("from_datum","to_datum")
  
  #automatically derive conversion for Villacarli
  raw_meas_file="../wq/qmeas_isab.txt" #table with raw measurements
  raw_meas = read.table(file=raw_meas_file, header=TRUE, sep="\t", stringsAsFactors=FALSE, na.strings=c("NA","?","na"))
  raw_meas=raw_meas[!is.na(raw_meas$stage_.m.),]
  gauge="Villacarli"
  valid_records=raw_meas$gauge==gauge
  datums=unique(raw_meas$stage_datum[valid_records])
  convert_info$n=NA
  single_row=convert_info[1,]
  
  for (from_datum in datums)  
  {
    for (to_datum in datums)  
    {
      
      if (from_datum==to_datum) next
      from_records=raw_meas$stage_datum[valid_records]==from_datum 
      to_records  =raw_meas$stage_datum[valid_records]==to_datum
      concom_dates = intersect(raw_meas$date[valid_records][from_records],
                               raw_meas$date[valid_records][to_records  ])
      if (length(concom_dates)==0) next
      merged=na.omit(merge( raw_meas[valid_records,c("date","stage_.m.")][from_records,],
                    raw_meas[valid_records,c("date","stage_.m.")][to_records  ,], by="date"))
      single_row[,]=NA #dummy row to fill
      single_row$gauge=gauge
      single_row$from_datum = from_datum
      single_row$to_datum   = to_datum
      mean_diff = mean(merged$stage_.m..x) - mean(merged$stage_.m..y)
      single_row$conversion2target_datum = paste0("x-",mean_diff)
      single_row$remarks="intercept only" 
      single_row$n=nrow(merged)
      old_entry= convert_info$gauge=="Villacarli" & 
              convert_info$from_datum == from_datum &
              convert_info$to_datum == to_datum
      convert_info=convert_info[!old_entry,]    #remove old entry
      convert_info=rbind(convert_info, single_row)
    }
  
  }  


  write.table(file="../wq/gauge_datum_conv.txt", convert_info, sep="\t", quote=FALSE, row.names=FALSE)
  #convert_info=read.table(file="../wq/gauge_datum_conv.txt", header=TRUE, sep="\t")
  

} 




convert_stage = function (gauge, convert_info, from_datum, to_datum, stage) {
  
  for (ifrom_datum in unique(from_datum)) #loop though multiple possible from_datums
  {  
    if (ifrom_datum == to_datum) next #nothing to do
    rel_records = from_datum == ifrom_datum
    x=stage[rel_records]
    
    conv_entry_i =convert_info$from_datum==ifrom_datum & 
      convert_info$to_datum  ==to_datum   & convert_info$gauge==gauge #relevant entry of table
    conversion = as.character(convert_info$conversion2target_datum[conv_entry_i])
    
    if (length(conversion)==0 || conversion=="" || conversion=="na") #no conversion found
    {  
      standard_datum =  as.character(convert_info$from_datum[convert_info$remarks=="standard" & convert_info$gauge==gauge])
      if (standard_datum %in% c(ifrom_datum, to_datum))
      {    
        stage[rel_records]=NA 
        warning(paste(" no conversion available from '", ifrom_datum,"' to '",to_datum,"'", sep=""))
      } else #take detour via standard datum 
      {  
        x_standard        = convert_stage(gauge=gauge, convert_info, from_datum=ifrom_datum   , to_datum=standard_datum, x)
        stage[rel_records]= convert_stage(gauge=gauge, convert_info, from_datum=standard_datum, to_datum=to_datum      , x_standard)
      }
    }  else
      stage[rel_records]=eval(parse(text=conversion)) #convert relevant stages
    
    within_range=  x >= convert_info$valid_from[conv_entry_i] & x <= convert_info$valid_to[conv_entry_i]
    stage[rel_records][!within_range]=NA #when range was specified, set those outside the range to NA
  }
  return(stage)
}
