###

# University of LLeida rainfall for CUASHI

###

setwd("U:/HiWi_ER/CUAHSI/UniLleida")

### Data reading and processing ####

UdL_prec <-  function(single_file){
  
  precip <- read.table(file=paste("RawData/", single_file, sep="") , stringsAsFactors = F, header=T, sep="\t", dec= ".") 
  colnames(precip) <- c("Fecha","Valor")
  
  precip$date      = as.POSIXct(strptime(precip$Fecha, format="%d/%m/%Y %H:%M", tz="etc/GMT-1"))
  
  #Contiuous times series 1min-resolution
  start_date = as.POSIXct(strptime(as.character(min(precip$date, na.rm=T)), "%Y-%m-%d %H:%M:%S", tz="etc/GMT-1"))  # nur WZ
  end_date = as.POSIXct(strptime(as.character(max(precip$date, na.rm=T)), "%Y-%m-%d %H:%M:%S", tz="etc/GMT-1"))  # nur WZ
  full_time = seq(from = start_date , to=end_date, by='min')
  
  full_data_min = data.frame(date=full_time, Valor=with(precip, Valor[match(full_time, date)]))
  full_data_min$Valor[which(is.na(full_data_min$Valor))] = 0
  
  #Quality management
  #Put times with noch rainfall to NA (with help of station comparison)
  
  if(single_file == "Roda_prec_2012_05_20.txt"){
    #Jos at data delivery: gap between 18.08.2011-03.03.2012
    full_data_min$Valor[which(as.character(full_data_min$date) == "2011-08-18 00:00:00") : which(as.character(full_data_min$date) == "2012-03-03 23:59:00")]=NA
    }
  
  if(single_file == "Villacarli_prec_2012_05_21.txt"){
    #station comparison with SAIH station show two more periods no rainfall recorded (see plots _old)
    full_data_min$Valor[which(as.character(full_data_min$date) == "2010-04-25 00:00:00") : which(as.character(full_data_min$date) == "2010-07-23 23:59:00")]=NA
    full_data_min$Valor[which(as.character(full_data_min$date) == "2011-04-26 00:00:00") : which(as.character(full_data_min$date) == "2011-06-17 23:59:00")]=NA
  }
  
  
  #Continuos time series 1hour-resolution (out of 1min series)
  full_data_min$date_full_hour =as.POSIXct(trunc(full_data_min$date, unit="hour"))
  full_data_min$Valor_new = full_data_min$Valor
  na_sum=function(x){ifelse(all(is.na(x)), NA, sum(x, na.rm=TRUE))}  #sum up disregarding NAs, except when all values are NA
  
  full_data_hour <-  aggregate(x = full_data_min$Valor_new, by=list(date=full_data_min$date_full_hour), FUN=na_sum) 
  full_data_min$date_full_hour = NULL
  full_data_min$Valor_new =NULL
  
  #Save as .RData
  statname=sub(pattern="+_.*",rep="", single_file) # get readout-date from file-name
  
  save(file = paste0("RData/",statname,"_min.RData"), list="full_data_min")  
  save(file = paste0("RData/",statname,"_hour.RData"), list="full_data_hour")  
  
  #Plots
  sub2006 <- subset(full_data_hour, format(date, "%Y")=="2006")
  sub2007 <- subset(full_data_hour, format(date, "%Y")=="2007")
  sub2008 <- subset(full_data_hour, format(date, "%Y")=="2008")
  sub2009 <- subset(full_data_hour, format(date, "%Y")=="2009")
  sub2010 <- subset(full_data_hour, format(date, "%Y")=="2010")
  sub2011 <- subset(full_data_hour, format(date, "%Y")=="2011")
  sub2012 <- subset(full_data_hour, format(date, "%Y")=="2012")
  
  pdf(paste("Plots/",statname,".pdf",sep = ""),width=10,height=8)
  
  if(nrow(full_data_hour) >0){
    plot(full_data_hour$date, full_data_hour$x, main=paste(statname), ylab = "Precipitation [mm/h]", xlab= "Time", type="h")
     legend("topleft", c("Range", as.character(round(range(full_data_hour$x, na.rm=T), digits = 1))))}
  
  cont_cunsum <- full_data_hour[-which(is.na(full_data_hour$x)),]
  plot(cont_cunsum$date, cumsum(cont_cunsum$x), pch=".",xlab="Time",ylab="Cumulative Sum")
  
  if(nrow(sub2006) >0){
    plot(sub2006$date, sub2006$x, main=paste(statname,"_2006",sep=""), ylab = "Precipitation [mm/h]", xlab= "Time", type="h")
    legend("topleft", c("Range", as.character(round(range(sub2011$x, na.rm=T), digits = 1))))}
 
   if(nrow(sub2007) >0){
    plot(sub2007$date, sub2007$x, main=paste(statname,"_2007",sep=""), ylab = "Precipitation [mm/h]", xlab= "Time", type="h")
    legend("topleft", c("Range", as.character(round(range(sub2007$x, na.rm=T), digits = 1))))}
 
   if(nrow(sub2008) >0){
    plot(sub2008$date, sub2008$x, main=paste(statname,"_2008",sep=""), ylab = "Precipitation [mm/h]", xlab= "Time", type="h")
    legend("topleft", c("Range", as.character(round(range(sub2008$x, na.rm=T), digits = 1))))}
  
  if(nrow(sub2009) >0){
    plot(sub2009$date, sub2009$x, main=paste(statname,"_2009",sep=""), ylab = "Precipitation [mm/h]", xlab= "Time", type="h")
    legend("topleft", c("Range", as.character(round(range(sub2009$x, na.rm=T), digits = 1))))}
  
  if(nrow(sub2010) >0){
    plot(sub2010$date, sub2010$x, main=paste(statname,"_2010",sep=""), ylab = "Precipitation [mm/h]", xlab= "Time", type="h")
    legend("topleft", c("Range", as.character(round(range(sub2010$x, na.rm=T), digits = 1))))}
  
  if(nrow(sub2011) >0){
    plot(sub2011$date, sub2011$x, main=paste(statname,"_2011",sep=""), ylab = "Precipitation [mm/h]", xlab= "Time", type="h")
    legend("topleft", c("Range", as.character(round(range(sub2011$x, na.rm=T), digits = 1))))}
  
  if(nrow(sub2012) >0){
    plot(sub2012$date, sub2012$x, main=paste(statname,"_2012",sep=""), ylab = "Precipitation [mm/h]", xlab= "Time", type="h")
    legend("topleft", c("Range", as.character(round(range(sub2012$x, na.rm=T), digits = 1))))}
  
  dev.off()
   
}

dir(path="RawData")
UdL_prec("Villacarli_prec_2012_05_21.txt")


### Station comparison ####

#UdL
load("RData/Roda_hour.RData")       ; Roda <- full_data_hour
load("RData/Villacarli_hour.RData") ; Villacarli <- full_data_hour

#SAIH-stations
load("U:/HiWi_ER/CUASHI/SAIH/R_Data/A047B85PQUIN_aggr.RData") ; A047_aggr <- precip_aggr
load("U:/HiWi_ER/CUASHI/SAIH/R_Data/P030B85PQUIN_aggr.RData") ; P030_aggr <- precip_aggr
load("U:/HiWi_ER/CUASHI/SAIH/R_Data/P031S85PQUIN_aggr.RData") ; P031_aggr <- precip_aggr
load("U:/HiWi_ER/CUASHI/SAIH/R_Data/P032S85PQUIN_aggr.RData") ; P032_aggr <- precip_aggr
load("U:/HiWi_ER/CUASHI/SAIH/R_Data/R063B85PQUIN_aggr.RData") ; R063_aggr <- precip_aggr
load("U:/HiWi_ER/CUASHI/SAIH/R_Data/R065B85PQUIN_aggr.RData") ; R065_aggr <- precip_aggr

station_compare_aggr <- function(data_3, data_4){
  
  pdf(paste("Plots/","station_compare_prec.pdf",sep = ""),width=10,height=8)
  
  # Cumsum-Plot station 1
  cont_cunsum_1 <- data_3
  nas <- which(is.na(data_3$x))
  if(length(nas) > 0){cont_cunsum_1 <- data_3[-which(is.na(data_3$x)),]}
  
  plot(cont_cunsum_1$date, cumsum(cont_cunsum_1$x), pch=".", main="Station 1",xlab="Time", ylab="Cumulative Sum")
  
  # Cumsum-Plot station 1
  cont_cunsum_2 <- data_4
  nas <- which(is.na(data_4$x))
  if(length(nas) > 0){cont_cunsum_2 <- data_4[-which(is.na(data_4$x)),]}
  
  plot(cont_cunsum_2$date, cumsum(cont_cunsum_2$x), pch=".", main="Station 2",xlab="Time", ylab="Cumulative Sum")
  
  # Cumsum-Comparison-Plot hourly continous data
  merge_data = merge(data_3, data_4, by = "date", all = FALSE) 
  # Removal NAs for cunsum plot
  nas1 = which(is.na(merge_data$x.x))
  nas2 = which(is.na(merge_data$x.y))
  if(length(nas1) > 0){merge_data = merge_data[-which(is.na(merge_data$x.x)),]}
  if(length(nas2) > 0){merge_data = merge_data[-which(is.na(merge_data$x.y)),]}
  
  #windows()
  plot(cumsum(merge_data$x.x), cumsum(merge_data$x.y), pch=".",  main= "Station comparison using cumulative sum", xlab="Station 1", ylab="Station 2")
  grid(5,5)
  abline(0,1, col="orange") 
  #identify(cumsum(merge_data$x.x),cumsum(merge_data$x.y), labels = merge_data$date)
  
  # Individual stations hourly continours data, merge()
  #windows()
  plot(merge_data$date, cumsum(merge_data$x.x), pch=".", ylim=c(0,max(c(cumsum(merge_data$x.x),cumsum(merge_data$x.y)))),main="Station comparison",xlab="Time", ylab="Cumulative Sum")
  points(merge_data$date, cumsum(merge_data$x.y), pch=".", col="red")
  points(merge_data$date, abs((cumsum(merge_data$x.x)-cumsum(merge_data$x.y))), pch=".", col="blue")
  legend("topleft",c("Station 1","Station 2","|Station 1 - Station 2|"), pch=19, col=c("black","red","blue"))
  #identify(cumsum(sub2014$VALOR.x), cumsum(sub2014$VALOR.y), labels = sub2014$POSIXDATE)
  
  dev.off()
}

station_compare_aggr(Villacarli,P032_aggr)


### CUASHI data ####

load("RData/Roda_min.RData")       ; cont_collected <- full_data_min ; statname <- "Roda"

load("RData/Villacarli_min.RData") ; cont_collected <- full_data_min ; statname <- "Villacarli"  

#cont_collected <- subset(cont_collected, format(posixdate, "%Y") >"2010")

n= nrow(cont_collected)

dtutc<- format(cont_collected$date, tz="UTC")
dtloc<- format(cont_collected$date, tz="Etc/GMT-1") # LocalDateTime winter time GMT+1
utc_offset = as.numeric(format(cont_collected$date, format = "%z"))/100
sitecode <- rep(statname,n)
varcode <- rep("rainfall_1",n)
censorc	<- rep("nc", n) 
methodc	<- rep("tipping_bucket_Campbell", n)
sourcec <- rep("UdL_meteo", n)
qualityc <- rep("1",n)

output_table=data.frame(DataValue=round(cont_collected$Valor, digits = 3),
                        ValueAccuracy="",
                        LocalDateTime=dtloc,
                        UTCOffset=utc_offset,
                        DateTimeUTC=dtutc,
                        SiteCode=sitecode,
                        VariableCode=varcode,
                        OffsetValue="",
                        OffsetTypeCode="",
                        CensorCode = censorc,
                        QualifierCode="",
                        MethodCode = methodc,
                        SourceCode = sourcec,
                        SampleCode="",
                        DerivedFromID="",
                        QualityControlLevelCode= qualityc )

r <- 700000 # max amount rows upload to CUASHI server is 750.000, but somehow not possible to upload; error message 'Bad Gateway'
k <- ceiling(n/r)-1 # number of necessary output tables (minus one)


for(i in 0:k){
  
  if(i < max(k)){
    output_table_write = output_table[(1+i*r):((i+1)*r),]
    write.table(file=paste0("CuashiData/",statname,"_Precip_", i+1,".csv"), x = output_table_write, na = "-9999", sep=",", row.names = FALSE, quote = FALSE)
  }else{
    output_table_write = output_table[(1+i*r):nrow(output_table),]
    write.table(file=paste0("CuashiData/",statname,"_Precip_", i+1,".csv"), x = output_table_write, na = "-9999", sep=",", row.names = FALSE, quote = FALSE)
  }
  if(i==max(k)){
    write.table(file=paste0("CuashiData/",statname,"_Precip.csv"), x = output_table, na = "-9999", sep=",", row.names = FALSE, quote = FALSE)
  }
}

### Final Check ####

final <- function(filename){
  
  cuashi_file = paste("CuashiData/", filename, sep="")
  cuashi <- read.csv(cuashi_file, header=T, na.strings="-9999", sep=",", dec="." )
  
  cuashi$LocalDateTime = as.POSIXct(strptime(cuashi$LocalDateTime, format="%Y-%m-%d %H:%M:%S", tz="etc/GMT-1"))
  
  
  #aggregate tips to full hour for plot
  cuashi$full_hour =as.POSIXct(trunc(cuashi$LocalDateTime, unit="hour"))
  
  na_sum=function(x){ifelse(all(is.na(x)), NA, sum(x, na.rm=TRUE))}  #sum up disregarding NAs, except when all values are NA
  cuashi_aggr <-  aggregate(x = cuashi$DataValue, by=list(date=cuashi$full_hour), FUN=na_sum)
  
  sub2006 <- subset(cuashi_aggr, format(date, "%Y")=="2006")
  sub2007 <- subset(cuashi_aggr, format(date, "%Y")=="2007")
  sub2008 <- subset(cuashi_aggr, format(date, "%Y")=="2008")
  sub2009 <- subset(cuashi_aggr, format(date, "%Y")=="2009")
  sub2010 <- subset(cuashi_aggr, format(date, "%Y")=="2010")
  sub2011 <- subset(cuashi_aggr, format(date, "%Y")=="2011")
  sub2012 <- subset(cuashi_aggr, format(date, "%Y")=="2012")
  filename <- cuashi$SiteCode[1]
  
  pdf(paste("Plots/",filename,"_Cuashi.pdf",sep = ""),width=10,height=8)
  
  if(nrow(cuashi_aggr) >0){
    plot(cuashi_aggr$date, cuashi_aggr$x, main=paste(filename), ylab = "Precipitation [mm/h]", xlab= "Time", type="h")
    #abline(v=as.numeric(cont_aggr$date[which(is.na(cont_aggr$x))]), lwd=1, col='red')
    legend("topleft", c("Range", as.character(round(range(cuashi_aggr$x, na.rm=T), digits = 1))))}
  
  cont_cunsum <- cuashi[-which(is.na(cuashi$DataValue)),]
  plot(cont_cunsum$LocalDateTime, cumsum(cont_cunsum$DataValue), pch=".",xlab="Time",ylab="Cumulative Sum")
  
  if(nrow(sub2006) >0){
    plot(sub2006$date, sub2006$x, main=paste(filename,"_2006",sep=""), ylab = "Precipitation [mm/h]", xlab= "Time", type="h")
    legend("topleft", c("Range", as.character(round(range(sub2011$x, na.rm=T), digits = 1))))}
  
  if(nrow(sub2008) >0){
    plot(sub2008$date, sub2008$x, main=paste(filename,"_2008",sep=""), ylab = "Precipitation [mm/h]", xlab= "Time", type="h")
    legend("topleft", c("Range", as.character(round(range(sub2008$x, na.rm=T), digits = 1))))}
  
  if(nrow(sub2009) >0){
    plot(sub2009$date, sub2009$x, main=paste(filename,"_2009",sep=""), ylab = "Precipitation [mm/h]", xlab= "Time", type="h")
    legend("topleft", c("Range", as.character(round(range(sub2009$x, na.rm=T), digits = 1))))}
  
  if(nrow(sub2010) >0){
    plot(sub2010$date, sub2010$x, main=paste(filename,"_2010",sep=""), ylab = "Precipitation [mm/h]", xlab= "Time", type="h")
    legend("topleft", c("Range", as.character(round(range(sub2010$x, na.rm=T), digits = 1))))}
  
  if(nrow(sub2011) >0){
    plot(sub2011$date, sub2011$x, main=paste(filename,"_2011",sep=""), ylab = "Precipitation [mm/h]", xlab= "Time", type="h")
    legend("topleft", c("Range", as.character(round(range(sub2011$x, na.rm=T), digits = 1))))}
  
  if(nrow(sub2012) >0){
    plot(sub2012$date, sub2012$x, main=paste(filename,"_2012",sep=""), ylab = "Precipitation [mm/h]", xlab= "Time", type="h")
    legend("topleft", c("Range", as.character(round(range(sub2012$x, na.rm=T), digits = 1))))}
  
  dev.off()
  
}

dir(path="CuashiData")
final("Villacarli.csv")
