###

# GFZ Villacarli data for Cuashi

###


setwd("U:/HiWi_ER/CUAHSI/GFZ_Villacarli") 

### Data collection ####

station_prefix="Villacarli" 

file_list <-  dir(path="DataFiles", pattern = station_prefix) # Create list of files from selected stations

cont_collected=data.frame()
n = 0 # to reocrd number of runs of loop

###


for (single_file in file_list) 
{
  
  print(paste("processing", single_file)) # prints in console which files processed
  
  if(single_file == "Villacarli_D1m_samp_2015_04_22_07_49_54_merge.dat"){
    cont <-  read.table(file=paste("DataFiles/", single_file, sep=""), skip=3, header=T, sep=",", dec=".", stringsAsFactors = FALSE, na.strings = c("null","NA")) # two empty first rows in data
    
  }else if(single_file == "Villacarli_D1m_samp_feb_apr_2013.txt"){
    
    cont_1 <-  read.table(file=paste("DataFiles/", single_file, sep=""), skip=1, nrow=94893, header=T, sep=",", dec=".", stringsAsFactors = FALSE, na.strings = c("null","NA")) #19 columns
    cont_1$Velocity =NULL #not in rest of data file
    cont_2 <-  read.table(file=paste("DataFiles/", single_file, sep=""), skip=94895, header=F, sep=",", dec=".", stringsAsFactors = FALSE, na.strings = c("null","NA")) #18 colums
    colnames(cont_2) = colnames(cont_1)
    cont = rbind(cont_1, cont_2)
    
  }else{
    cont <-  read.table(file=paste("DataFiles/", single_file, sep=""), skip=1, header=T, sep=",", dec=".", stringsAsFactors = FALSE, na.strings = c("null","NA"))
  }
  
  if(cont[1,1] == "TS"){cont = cont[-1,]}
  if(cont[1,1] == ""){cont = cont[-1,]}
  
  format="%Y-%m-%d %H:%M:%S" 
  cont$date=as.POSIXct(strptime(cont[,1], format=format, tz="Etc/GMT-1")) #compute date ("Etc/GMT-1" stands for wintertime (GMT+1)without daylight saving) I dunno why it is "-1" instead of "+1!
  
  #Select columns here
  cont <-  data.frame(date=cont$date, Rain1_Tot=as.numeric(cont$Rain1_Tot), Rain2_Tot=as.numeric(cont$Rain2_Tot), 
                      AirT_107=as.numeric(cont$AirT_107),WatT_107=as.numeric(cont$WatT_107))
  
  duplicates_1 = which(duplicated(cont$date))
  if(length(duplicates_1) >0)
    cont = cont[-duplicates_1,]
  
  duplicates_2 = cont$date %in% cont_collected$date #duplicates accross data files
  if(length(duplicates_2) >0)
    cont = cont[!duplicates_2,]
  
  # Merge datasets
  cont_collected <-  rbind(cont_collected, cont) # collect all single files from loop
  
  sorted_ix=sort.int(cont_collected$date, index.return = TRUE)$ix
  cont_collected = cont_collected[sorted_ix,]
  
  n <- n+1
  
  if(n == length(file_list)) # all files processed
  {
    #Save as .RData
    save(file = paste0("MergeData/",station_prefix,"_merged.RData"), list="cont_collected")  
  }
  
}


### Data processing ####

#sensor: "Rain1_Tot", "Rain2_Tot", "AirT_107", "WatT_107"

gfz_vill <- function(sensor){
  
  load("MergeData/Villacarli_merged.RData") ; data <- cont_collected
  
  data <-  data.frame(date=data$date, x= data[,which(colnames(data) == sensor)])
  
  # Corrections
  if(sensor == "Rain1_Tot"){
    data$x[which(data$date > as.POSIXct("2014-08-20 03:00:00", tz="Etc/GMT-1") & data$date < as.POSIXct("2014-08-20 10:00:00",tz="Etc/GMT-1"))] <- NA
    data$x[which(data$date > as.POSIXct("2015-03-20 21:00:00", tz="Etc/GMT-1") & data$date < as.POSIXct("2015-03-25 23:00:00",tz="Etc/GMT-1"))] <- NA
    
    
    #data$x[which(as.character(data$date)== "2014-08-20 03:00:00") : which(as.character(data$date) == "2014-08-20 10:00:00")] <- NA
    #data$x[which(as.character(data$date)== "2015-03-20 21:00:00") : which(as.character(data$date) == "2015-03-25 23:00:00")] <- NA
  }
  
  if(sensor == "Rain2_Tot"){
    data$x[which(data$date > as.POSIXct("2014-04-19 22:00:00", tz="Etc/GMT-1") & data$date < as.POSIXct("2014-04-21 22:00:00", tz="Etc/GMT-1"))] <- NA
    data$x[which(data$date > as.POSIXct("2014-05-25 16:00:00", tz="Etc/GMT-1") & data$date < as.POSIXct("2014-05-26 16:00:00", tz="Etc/GMT-1"))] <- NA
    data$x[which(data$date > as.POSIXct("2014-09-11 15:00:00", tz="Etc/GMT-1") & data$date < as.POSIXct("2014-09-11 17:00:00", tz="Etc/GMT-1"))] <- NA
    data$x[which(data$date > as.POSIXct("2014-11-14 06:00:00", tz="Etc/GMT-1") & data$date < as.POSIXct("2014-11-17 06:00:00", tz="Etc/GMT-1"))] <- NA
    data$x[which(data$date > as.POSIXct("2016-01-09 04:00:00", tz="Etc/GMT-1") & data$date < as.POSIXct("2016-01-11 04:00:00", tz="Etc/GMT-1"))] <- NA
    
    #data$x[which(as.character(data$date)== "2014-04-19 22:00:00") : which(as.character(data$date) == "2014-04-21 22:00:00")] <- NA
    #data$x[which(as.character(data$date)== "2014-05-25 16:00:00") : which(as.character(data$date) == "2014-05-26 16:00:00")] <- NA
    #data$x[which(as.character(data$date)== "2014-09-11 15:00:00") : which(as.character(data$date) == "2014-09-11 17:00:00")] <- NA
    #data$x[which(as.character(data$date)== "2014-11-14 06:00:00") : which(as.character(data$date) == "2014-11-17 06:00:00")] <- NA
    #data$x[which(as.character(data$date)== "2016-01-09 04:00:00") : which(as.character(data$date) == "2016-01-11 04:00:00")] <- NA
  }
  
  if(sensor == "TSS_C1" | sensor == "TSS_C2" | sensor == "TSS"){
    data$x[which(data$x < 0)] <- NA
  }
  
  #aggregate to full hour
  data$date_full_hour =as.POSIXct(trunc(data$date, unit="hour"))
  na_sum=function(x){ifelse(all(is.na(x)), NA, sum(x, na.rm=TRUE))}  #sum up disregarding NAs, except when all values are NA
  if(sensor == "Rain1_Tot" | sensor == "Rain2_Tot"){data_aggr <-  aggregate(x = data$x, by=list(date=data$date_full_hour), FUN=na_sum)}
  if(sensor == "AirT_107" | sensor == "WatT_107"){data_aggr <-  aggregate(x = data$x, by=list(date=data$date_full_hour), FUN=mean)}
  #include for others
  data$date_full_hour = NULL
  
  #Save as .RData
  filename <- "Villacarli_D1m" 
  save(file = paste0("MergeData/",filename,"_",sensor,"_merged.RData"), list="data")  
  save(file = paste0("MergeData/",filename,"_",sensor,"_merged_aggr.RData"), list="data_aggr") 
  
  sub2011 <- subset(data_aggr, format(date, "%Y")=="2011")
  sub2012 <- subset(data_aggr, format(date, "%Y")=="2012")
  sub2013 <- subset(data_aggr, format(date, "%Y")=="2013")
  sub2014 <- subset(data_aggr, format(date, "%Y")=="2014")
  sub2015 <- subset(data_aggr, format(date, "%Y")=="2015")
  sub2016 <- subset(data_aggr, format(date, "%Y")=="2016")
  filename <- "Villacarli_D1m" 
  
  # detection beginning and ending of big data gaps
  
  #time_diff <- diff(data_aggr$date)
  #plot(data_aggr$date, data_aggr$x, main=paste(filename), ylab = sensor, xlab= "Time", type="l")
  #points(data_aggr$date[which(test>24*14)], data_aggr$x[which(test>24*14)], col="red")
  #points(data_aggr$date[which(test>24*14)+1], data_aggr$x[which(test>24*14)+1], col="green")
  
  #data_aggr[which(test>24*14),] # start gap
  #data_aggr[which(test>24*14)+1,] # end gap
  
  pdf(paste("Plots/",filename,"_",sensor,".pdf",sep = ""),width=10,height=8)
  
  if(nrow(data_aggr) >0){
    plot(data_aggr$date, data_aggr$x, main=paste(filename), ylab = sensor, xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(data_aggr$x, na.rm=T), digits = 1))))}
  
  if(sensor == "Rain1_Tot" | sensor == "Rain2_Tot"){
    cont_cunsum <- data_aggr
    nas <- which(is.na(data_aggr$x))
    if(length(nas) > 0){cont_cunsum <- data_aggr[-which(is.na(data_aggr$x)),]}
    
    plot(cont_cunsum$date, cumsum(cont_cunsum$x), pch=".",xlab="Time",ylab="Cumulative Sum")
  }
  
  if(nrow(sub2011) >0){
    plot(sub2011$date, sub2011$x, main=paste(filename,"_2011",sep=""), ylab = sensor, xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(sub2011$x, na.rm=T), digits = 1))))}
  
  if(nrow(sub2012) >0){
    plot(sub2012$date, sub2012$x, main=paste(filename,"_2012",sep=""), ylab = sensor, xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(sub2012$x, na.rm=T), digits = 1))))}
  
  if(nrow(sub2013) >0){
    plot(sub2013$date, sub2013$x, main=paste(filename,"_2013",sep=""), ylab = sensor, xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(sub2013$x, na.rm=T), digits = 1))))}
  
  if(nrow(sub2014) >0){
    plot(sub2014$date, sub2014$x, main=paste(filename,"_2014",sep=""), ylab = sensor, xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(sub2014$x, na.rm=T), digits = 1))))}
  
  if(nrow(sub2015) >0){
    plot(sub2015$date, sub2015$x, main=paste(filename,"_2015",sep=""), ylab = sensor, xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(sub2015$x, na.rm=T), digits = 1))))}
  
  if(nrow(sub2016) >0){
    plot(sub2016$date, sub2016$x, main=paste(filename,"_2016",sep=""), ylab = sensor, xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(sub2016$x, na.rm=T), digits = 1))))}
  
  dev.off()
}

gfz_vill("Rain2_Tot")

###



### Precipitation: Station comparison ####

load("MergeData/Villacarli_D1m_Rain1_Tot_merged_aggr.RData") ; ViaGFZ1_aggr <- data_aggr
load("MergeData/Villacarli_D1m_Rain2_Tot_merged_aggr.RData") ; ViaGFZ2_aggr <- data_aggr

#SESAM stations
load("U:/HiWi_ER/CUAHSI/SESAM/MergeData/Via1_merged_aggr.RData") ; Via1_aggr <- cont_collected_aggr
load("U:/HiWi_ER/CUAHSI/SESAM/MergeData/Via2_merged_aggr.RData") ; Via2_aggr <- cont_collected_aggr


#Function just using prior aggregated data
station_compare_aggr <- function(data_3, data_4){
  
  pdf(paste("Plots/","station_compare_aggr.pdf",sep = ""),width=10,height=8)
  
  # Cumsum-Plot station 1
  cont_cunsum_1 <- data_3
  nas <- which(is.na(data_3$x))
  if(length(nas) > 0){cont_cunsum_1 <- data_3[-which(is.na(data_3$x)),]}
  
  plot(cont_cunsum_1$date, cumsum(cont_cunsum_1$x), pch=".", main="Station 1",sub="single files aggregated hourly, continous times series",xlab="Time", ylab="Cumulative Sum")
  
  # Cumsum-Plot station 2
  cont_cunsum_2 <- data_4
  nas <- which(is.na(data_4$x))
  if(length(nas) > 0){cont_cunsum_2 <- data_4[-which(is.na(data_4$x)),]}
  
  plot(cont_cunsum_2$date, cumsum(cont_cunsum_2$x), pch=".", main="Station 2",sub="single files aggregated hourly, continous times series",xlab="Time", ylab="Cumulative Sum")
  
  # Cumsum-Comparison-Plot hourly continous data
  merge_data = merge(data_3, data_4, by = "date", all = FALSE) 
  # Removal NAs for cunsum plot
  nas1 = which(is.na(merge_data$x.x))
  if(length(nas1) > 0){merge_data = merge_data[-which(is.na(merge_data$x.x)),]}
  nas2 = which(is.na(merge_data$x.y))
  if(length(nas2) > 0){merge_data = merge_data[-which(is.na(merge_data$x.y)),]}
  
  #windows()
  plot(cumsum(merge_data$x.x), cumsum(merge_data$x.y), pch=".",  main= "Station comparison using cumulative sum",sub="single files aggregated hourly,continous times series; merge (... all=FALSE);", xlab="Station 1", ylab="Station 2")
  grid(5,5)
  abline(0,1, col="orange") 
  #identify(cumsum(merge_data$x.x),cumsum(merge_data$x.y), labels = merge_data$date)
  
  # Individual stations hourly continours data, merge()
  #windows()
  plot(merge_data$date, cumsum(merge_data$x.x), pch=".", ylim=c(0,max(c(cumsum(merge_data$x.x),cumsum(merge_data$x.y)))),main="Station comparison",sub="single files aggregated hourly, continous times series; merge (... all=FALSE)",xlab="Time", ylab="Cumulative Sum")
  points(merge_data$date, cumsum(merge_data$x.y), pch=".", col="red")
  points(merge_data$date, abs((cumsum(merge_data$x.x)-cumsum(merge_data$x.y))), pch=".", col="blue")
  legend("topleft",c("Station 1","Station 2","|Station 1 - Station 2|"), pch=19, col=c("black","red","blue"))
  #identify(cumsum(sub2014$VALOR.x), cumsum(sub2014$VALOR.y), labels = sub2014$POSIXDATE)
  
  dev.off()
}

station_compare_aggr(ViaGFZ1_aggr,Via2_aggr) # data_3/4 aggregated data

### Precipitation: CUASHI data ####

source("U:/HiWi_ER/CUAHSI/f_write4cuahsi.R")

load("MergeData/Villacarli_D1m_Rain1_Tot_merged.RData") ; statname <- "Villacarli_Bridge_1" 

load("MergeData/Villacarli_D1m_Rain2_Tot_merged.RData") ; statname <- "Villacarli_Bridge_2"

colnames(data) = c("date", "DataValue")

write4cuahsi(cdata=data, dest_file=paste(statname,"_precip",sep=""), sitecode=statname, varcode="rainfall_1" , methodc="tipping_bucket_Thies" , sourcec="SESAMII_meteo", censorc="nc", OffsetValue="", qualityc=1, ValueAccuracy="", maxrec=700000)

### Precipitation: Final Check ####

final <- function(filename){
  
  cuashi_file = paste("CuahsiData/", filename, sep="")
  cuashi <- read.csv(cuashi_file, header=T, na.strings="-9999", sep=",", dec="." )
  
  cuashi$LocalDateTime = as.POSIXct(strptime(cuashi$LocalDateTime, format="%Y-%m-%d %H:%M:%S", tz="etc/GMT-1"))
  
  
  #aggregate tips to full hour for plot
  cuashi$full_hour =as.POSIXct(trunc(cuashi$LocalDateTime, unit="hour"))
  
  na_sum=function(x){ifelse(all(is.na(x)), NA, sum(x, na.rm=TRUE))}  #sum up disregarding NAs, except when all values are NA
  cuashi_aggr <-  aggregate(x = cuashi$DataValue, by=list(date=cuashi$full_hour), FUN=na_sum)
  
  sub2011 <- subset(cuashi_aggr, format(date, "%Y")=="2011")
  sub2012 <- subset(cuashi_aggr, format(date, "%Y")=="2012")
  sub2013 <- subset(cuashi_aggr, format(date, "%Y")=="2013")
  sub2014 <- subset(cuashi_aggr, format(date, "%Y")=="2014")
  sub2015 <- subset(cuashi_aggr, format(date, "%Y")=="2015")
  sub2016 <- subset(cuashi_aggr, format(date, "%Y")=="2016")
  filename <- cuashi$SiteCode[1]
  
  pdf(paste("Plots/",filename,"_Cuashi.pdf",sep = ""),width=10,height=8)
  
  if(nrow(cuashi_aggr) >0){
    plot(cuashi_aggr$date, cuashi_aggr$x, main=paste(filename), ylab = "Precipitation [mm/h]", xlab= "Time", type="h")
    #abline(v=as.numeric(cont_aggr$date[which(is.na(cont_aggr$x))]), lwd=1, col='red')
    legend("topleft", c("Range", as.character(round(range(cuashi_aggr$x, na.rm=T), digits = 1))))}
  
  cont_cunsum <- cuashi[-which(is.na(cuashi$DataValue)),]
  plot(cont_cunsum$LocalDateTime, cumsum(cont_cunsum$DataValue), pch=".",xlab="Time",ylab="Cumulative Sum")
  
  if(nrow(sub2011) >0){
    plot(sub2011$date, sub2011$x, main=paste(filename,"_2011",sep=""), ylab = "Precipitation [mm/h]", xlab= "Time", type="h")
    #abline(v=as.numeric(sub2011$date[which(is.na(sub2011$x))]), lwd=1, col='red')
    legend("topleft", c("Range", as.character(round(range(sub2011$x, na.rm=T), digits = 1))))}
  
  if(nrow(sub2012) >0){
    plot(sub2012$date, sub2012$x, main=paste(filename,"_2012",sep=""), ylab = "Precipitation [mm/h]", xlab= "Time", type="h")
    legend("topleft", c("Range", as.character(round(range(sub2012$x, na.rm=T), digits = 1))))}
  
  if(nrow(sub2013) >0){
    plot(sub2013$date, sub2013$x, main=paste(filename,"_2013",sep=""), ylab = "Precipitation [mm/h]", xlab= "Time", type="h")
    legend("topleft", c("Range", as.character(round(range(sub2013$x, na.rm=T), digits = 1))))}
  
  if(nrow(sub2014) >0){
    plot(sub2014$date, sub2014$x, main=paste(filename,"_2014",sep=""), ylab = "Precipitation [mm/h]", xlab= "Time", type="h")
    legend("topleft", c("Range", as.character(round(range(sub2014$x, na.rm=T), digits = 1))))}
  
  if(nrow(sub2015) >0){
    plot(sub2015$date, sub2015$x, main=paste(filename,"_2015",sep=""), ylab = "Precipitation [mm/h]", xlab= "Time", type="h")
    legend("topleft", c("Range", as.character(round(range(sub2015$x, na.rm=T), digits = 1))))}
  
  if(nrow(sub2016) >0){
    plot(sub2016$date, sub2016$x, main=paste(filename,"_2016",sep=""), ylab = "Precipitation [mm/h]", xlab= "Time", type="h")
    legend("topleft", c("Range", as.character(round(range(sub2016$x, na.rm=T), digits = 1))))}
  
  dev.off()
  
}

dir(path="CuahsiData")
final("Villacarli_Bridge_2_precip.csv")


### Temeperature: Quality control ####


load("MergeData/Villacarli_D1m_AirT_107_merged_aggr.RData") ; ViaGFZ <- data_aggr
load("U:/HiWi_ER/CUAHSI/SAIH/R_Data/A145B01TEMPE.RData") ; A145 <- temp


temp_func <- function(data_1, data_2){
  
  pdf(paste("plots/station_compare_temp.pdf",sep = ""),width=10,height=8)
  
  #aggregate to full hour
  data_2$date_full_hour = as.POSIXct(trunc(data_2$posixdate, unit="hour"))
  na_sum=function(x){ifelse(all(is.na(x)), NA, sum(x, na.rm=TRUE))}  #sum up disregarding NAs, except when all values are NA
  data_2 <-  aggregate(x = data_2$Valor, by=list(date=data_2$date_full_hour), FUN=mean)
  
  merge_data = merge(data_1, data_2, by = "date", all = FALSE) 
  
  hist(merge_data$x.x - merge_data$x.y, col=2, breaks=50)
  
  plot(merge_data$x.x, merge_data$x.y, pch=19, xlab = "data_1", ylab = "data_2", main = "Station comparison", 
       xlim = c(min(merge_data[,c(3,2)], na.rm = T),max(merge_data[,c(3,2)], na.rm = T)),
       ylim = c(min(merge_data[,c(3,2)], na.rm = T),max(merge_data[,c(3,2)], na.rm = T))
  )
  #abline(-mean(merge_data$x.x - merge_data$x.y, na.rm = T),1, col="red",lwd=2)
  #abline(-mean(merge_data$x.x - merge_data$x.y, na.rm = T)+10,1, col="red",lwd=2)
  #abline(-mean(merge_data$x.x - merge_data$x.y, na.rm = T)-10,1, col="red",lwd=2)
  # abline(0,1, col="blue",lwd=2)
  
  #windows()
  plot(merge_data$date,(merge_data$x.x - merge_data$x.y), pch=".")
  #points(ind$date,ind$index*10, pch=19, col=2, cex=1)
  #identify(merge_data$date, (merge_data$x.x - merge_data$x.y),labels = merge_data$date) 
  
  diff_temp <- diff(data_1$x)
  diff_time <- diff(data_1$date)
  indic = abs(diff_temp) /as.numeric(diff_time)
  indic[which(indic < 5)] <- NA
  
  #windows()
  plot(data_1$date[-1],indic, ylim=c(4,max(indic, na.rm=T)), pch=19, xlab="Time", ylab="Temperature Index", main="Station 1: Temperature Quality Index", sub="Index = abs(diff_temp) / diff_time")
  abline(h=5, col=2, lwd=2)
  #identify(data_1$date[-1], indic,labels = data_1$date[-1]) 
  
  diff_temp <- diff(data_2$x)
  diff_time <- diff(data_2$date)
  indic = abs(diff_temp) /as.numeric(diff_time)
  indic[which(indic < 5)] <- NA
  ind <- data.frame(date=data_2$date[-1], index = indic)
  
  #windows()
  plot(data_2$date[-1],indic, ylim=c(4,max(indic, na.rm=T)), pch=19, xlab="Time", ylab="Temperature Index", main="Station 2: Temperature Quality Index", sub="Index = abs(diff_temp) / diff_time")
  abline(h=5, col=2, lwd=2)
  #identify(data_2$date[-1], indic,labels = data_2$date[-1]) 
  
  dev.off()
}

temp_func(ViaGFZ, A145) #data_1 has to be ViaGFZ; data_2 15_min resolution data from SAIH

### Temperature: CUAHSI data ####

source("U:/HiWi_ER/CUAHSI/f_write4cuahsi.R")

load("MergeData/Villacarli_D1m_AirT_107_merged.RData") ; statname <- "Villacarli_Bridge" 

colnames(data) = c("date", "DataValue")

write4cuahsi(cdata=data, dest_file=paste(statname,"_temp",sep=""), sitecode=statname, varcode="temperature_1" , methodc="temperature_Campbell" , sourcec="SESAMII_meteo", censorc="nc", OffsetValue="", qualityc=1, ValueAccuracy="", maxrec=700000)

### Temperature: Final check####


final <- function(filename){
  
  cuashi_file = paste("CuahsiData/", filename, sep="")
  cuashi <- read.csv(cuashi_file, header=T, na.strings="-9999", sep=",", dec="." )
  
  cuashi$LocalDateTime = as.POSIXct(strptime(cuashi$LocalDateTime, format="%Y-%m-%d %H:%M:%S", tz="etc/GMT-1"))
  
  #aggregate to full hour
  cuashi$date_full_hour = as.POSIXct(trunc(cuashi$LocalDateTime, unit="hour"))
  na_sum=function(x){ifelse(all(is.na(x)), NA, sum(x, na.rm=TRUE))}  #sum up disregarding NAs, except when all values are NA
  cuashi <-  aggregate(x = cuashi$DataValue, by=list(date=cuashi$date_full_hour), FUN=mean)
  colnames(cuashi) <- c("LocalDateTime", "DataValue")
  
  sub2008 <- subset(cuashi, format(LocalDateTime, "%Y")=="2008")
  sub2009 <- subset(cuashi, format(LocalDateTime, "%Y")=="2009")
  sub2010 <- subset(cuashi, format(LocalDateTime, "%Y")=="2010")
  sub2011 <- subset(cuashi, format(LocalDateTime, "%Y")=="2011")
  sub2012 <- subset(cuashi, format(LocalDateTime, "%Y")=="2012")
  sub2013 <- subset(cuashi, format(LocalDateTime, "%Y")=="2013")
  sub2014 <- subset(cuashi, format(LocalDateTime, "%Y")=="2014")
  sub2015 <- subset(cuashi, format(LocalDateTime, "%Y")=="2015")
  sub2016 <- subset(cuashi, format(LocalDateTime, "%Y")=="2016")
  
  pdf(paste("plots/",filename,"_Cuashi",".pdf",sep = ""),width=10,height=8)
  
  if(nrow(cuashi) >0){
    plot(cuashi$LocalDateTime, cuashi$DataValue, main=paste(filename), ylab = "Temperature C", xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(cuashi$DataValue, na.rm=T), digits = 1))))}
  
  if(nrow(sub2008) >0){
    plot(sub2008$LocalDateTime, sub2008$DataValue, main=paste(filename,"_2008",sep=""), ylab = "Temperature C", xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(sub2008$DataValue, na.rm=T), digits = 1))))}
  
  if(nrow(sub2009) >0){
    plot(sub2009$LocalDateTime, sub2009$DataValue, main=paste(filename,"_2009",sep=""), ylab = "Temperature C", xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(sub2009$DataValue, na.rm=T), digits = 1))))}
  
  if(nrow(sub2010) >0){
    plot(sub2010$LocalDateTime, sub2010$DataValue, main=paste(filename,"_2010",sep=""), ylab = "Temperature C", xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(sub2010$DataValue, na.rm=T), digits = 1))))}
  
  if(nrow(sub2011) >0){
    plot(sub2011$LocalDateTime, sub2011$DataValue, main=paste(filename,"_2011",sep=""), ylab = "Temperature C", xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(sub2011$DataValue, na.rm=T), digits = 1))))}
  
  if(nrow(sub2012) >0){
    plot(sub2012$LocalDateTime, sub2012$DataValue, main=paste(filename,"_2012",sep=""), ylab = "Temperature C", xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(sub2012$DataValue, na.rm=T), digits = 1))))}
  
  if(nrow(sub2013) >0){
    plot(sub2013$LocalDateTime, sub2013$DataValue, main=paste(filename,"_2013",sep=""), ylab = "Temperature C", xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(sub2013$DataValue, na.rm=T), digits = 1))))}
  
  if(nrow(sub2014) >0){
    plot(sub2014$LocalDateTime, sub2014$DataValue, main=paste(filename,"_2014",sep=""), ylab = "Temperature C", xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(sub2014$DataValue, na.rm=T), digits = 1))))}
  
  if(nrow(sub2015) >0){
    plot(sub2015$LocalDateTime, sub2015$DataValue, main=paste(filename,"_2015",sep=""), ylab = "Temperature C", xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(sub2015$DataValue, na.rm=T), digits = 1))))}
  
  if(nrow(sub2016) >0){
    plot(sub2016$LocalDateTime, sub2016$DataValue, main=paste(filename,"_2016",sep=""), ylab = "Temperature C", xlab= "Time", type="l")
    legend("topleft", c("Range", as.character(round(range(sub2016$DataValue, na.rm=T), digits = 1))))}
  
  dev.off()
  
}

dir(path="CuahsiData")

final("Villacarli_Bridge_Temp.csv")




