###

# SESAM precipitation data for CUASHI

###

setwd("U:/HiWi_ER/CUAHSI/SESAM")


### Content ####
# Step1: Data collection and processing

  # loop to process and merge data files of SESAM stations
  # single files marked with NA at beginning and ending
  # tips due to reading process, calibration correction errors removed
  # some files wrong time zone in header => time corrected here
  # Plot: Precipitation [mm/h] vs. Time (in folder 'Plots')
  # Export merged data as .RData to folder 'MergeData'; tipping point and hourly values (_aggr)

# Step 2: Station comparison

  # function using exported data from Step1 to compare station
  # Cumsum-Plots
  # Plots saved in folder 'Plots'
  # If problem detected back to Step1

# Step 3: CUASHI Data

  # Export table in for Cuashi
  # all data in GMT+1
  # tables saved in folder 'CuahsiData'



### Step 1: Data collection and processing ####
    

###

station_prefix="Cabecera1"
station_id_golm="Cab1"

station_prefix="Cabecera2" 
station_id_golm="Cab2"

station_prefix="Villacarli2" 
station_id_golm="Via2"

station_prefix="Ceguera2" 
station_id_golm="Ceg2"

station_prefix="Carrasquero2" 
station_id_golm="Car"

station_prefix="Lascuarre1" 
station_id_golm="Las1"

station_prefix="Lascuarre2"
station_id_golm="Las2"

station_prefix="Badland" 
station_id_golm="Via1"
 
station_prefix="Ceguera1" 
station_id_golm="Ceg1"

station_prefix="Lascuarre3" 
station_id_golm="Lascuarre_tube"

###

tipping_volume <- 0.202 # for all but Lascuarre3!
#tipping_volume <- 0.105 # for Lascuarre3

file_list <-  dir(path="DataFiles", pattern = station_prefix) # Create list of files from selected stations
cont_collected=data.frame()
cont_collected_aggr=data.frame()

p_total=0 

date_min=date_max=NA 

n = 0 # to reocrd number of runs of loop

###

if (length(file_list)==0) stop("No files found")

for (single_file in file_list) 
{
  print(paste("processing", single_file)) # prints in console which files processed
  
 # Read data, edit event-column 
  
  cont <-  read.table(file=paste("DataFiles/", single_file, sep=""), skip=1, header=TRUE, sep="\t", stringsAsFactors = FALSE, 
                    na.strings = c("null","NA")) # read file from wd
  
  names(cont)[grepl(names(cont), pattern = "Event|Ereignis")]="Event" # rename column with 'Event' or 'Ereignis' in name to 'Event'
  cont$Event <-  sub(pattern = "[,\\. ]00$", rep="", x=cont$Event) #remove non-integer part of events (two zeros that are in raw data)
  cont$Event <-  sub(pattern = "[,\\.]", rep="", x=cont$Event) #remove group separator , or . that is in raw data
  
  cont=rbind(cont[1,], cont) # duplicate first row to make sure to keep the start of series there
  if (min(as.numeric(cont$Event), na.rm=T) == 0)
    cont$Event[1] <-  0 else # 0 no tip, from reading process
    cont$Event[1] <-  min(as.numeric(cont$Event), na.rm=T)-1 # some start with NA; when temperature data would be removed later; furthermore difference formed later => -1 to get one tick for first event;
  
  cont=rbind(cont, cont[nrow(cont),]) #duplicate last line to ensure to keep the ending of time series
  cont$Event[nrow(cont)]=cont$Event[max(which(!is.na(cont$Event)))]
  
 ### Process 'Protokoliert' and 'Logged' entries in data ##
  logger_connected <- c() # creat empty vector 'logger_connected'
  
  for (i in 4:ncol(cont))
    logger_connected <-  c(logger_connected, which((cont[,i] %in% c("Protokolliert","Logged")))) # search for all rows with 'Protokolliert' or 'Loggeed'
  
  logger_connected <- setdiff(logger_connected, c(1,nrow(cont))) # first and last (created before) no to be removed
  if (any(logger_connected))
    cont=cont[-logger_connected,] #remove logger connection events (except in first and last row)
  
  ### Temperature data
  temp_col=which(grepl(names(cont), pattern = "Temp")) #check if there is a column with temperature
  
  if (length(temp_col) >0) # if there is a column with 'Temp'...
  {
    names(cont)[temp_col]="Temp" # rename columns that have 'Temp' in their name to 'Temp'
    temp_only_recs = is.na(cont$Event) & !is.na(cont$Temp) # records (rows) containing temperature only
    cont=cont[!temp_only_recs,] # removes rows with only temperature record
    cont$Temp=NULL # remove temperature column
  }
  
  
  ### Removal undefinable entries in event column
  cont$Event[is.na(cont$Event)]=-1 # mask real NAs as -1
  cont$Event = as.numeric(cont$Event) 
  cont = cont[!is.na(cont$Event),]
  cont$Event[cont$Event<0]=NA  #restore true NAs
  
  
 ### Tippings, no-rainfall situation
  cont$tips = diff(c(cont$Event[1],cont$Event)) # when event number more often (reading process), then no tipping
  
  # Cover no-rainfall situation before first and after last recorded tip
  cont$tips[1] <-  NA # first entry no rainfall; indicating beginning time series
  cont$tips[nrow(cont)] <- NA # last entry no rainfall; indicating end time series
  
  if(length(which(cont$tips == 0)) > 0) # if there are entries 0 (artefact from reading process), remove them
  cont <- cont[-which(cont$tips == 0),]
  
  cont$tips = cont$tips * tipping_volume # convert tips into amount [mm]
  
  
 ### Date-Format-Management
  
  # Infer date format in file (depends on system HoboWare settings during export)
  file_date=sub(pattern="[^_]+_",rep="", single_file) # get readout-date from file-name
  file_date=sub(pattern=".+(\\d{4}_\\d{2}_\\d{2}).*",rep="\\1", single_file, perl = TRUE) # just date
  file_date= as.POSIXct(strptime(file_date, format = "%Y_%m_%d", tz="Etc/GMT-1")) # transfer to time-format
  readout_year=as.numeric(format(file_date,"%y")) # get year of readout
  
  digits = sapply(X = cont[,2], FUN = strsplit, split="\\.", simplify = TRUE) # date column (still character), split by .
  digits = matrix(unlist(digits), ncol = 3, byrow = TRUE) # create matric with three column (year - month - day/time)
  
  if (grepl(digits[1,3], pattern=" ")) # check if time in day column
  {
    parts = sapply(X = digits[,3], FUN = strsplit, split=" ", simplify = TRUE) # separate day and time
    parts = matrix(unlist(parts), ncol = 2, byrow = TRUE) # matrix with day and time column
    digits[,3] = parts[,1] # column in digits matrix for day filled with day
    cont[,2] = apply(digits, MARGIN = 1, paste, collapse=".") # create  date and time column as expected by subsequent script
    cont=cbind(cont[1:2], time=parts[,2], cont[3:ncol(cont)])
  }
  
  # Get date formate of data
  format=""
  for(i in 1:3)
  {  
    # Step 1: Year-column
    if(!grepl(pattern = "[yY]", x=format) & all(as.numeric(digits[,i])>1900)) 
    {format=paste0(format,"%Y."); next} 
    if(i==1 & diff(range(as.numeric(digits[,i])))<3 & all(as.numeric(digits[,i])>09) & all(digits[,i]<= readout_year) & all(digits[,i]>= readout_year-1)) 
    {format=paste0(format,"%y."); next}
    # Step 2: Day-column
    if(!grepl(pattern = "[d]", x=format) & all(as.numeric(digits[,i])<=31) & any(as.numeric(digits[,i])>12)) 
    {format=paste0(format,"%d."); next}
    # Step 3: Month-column
    if(!grepl(pattern = "[m]", x=format) & all(as.numeric(digits[,i])<=12) ) 
    {format=paste0(format,"%m."); next}
    # Step 4: Unidentified column
    if (i==3)      #unidentified  column, use remainder, if possible
    {
      if(!grepl(pattern = "[yY]", x=format))
      {format=paste0(format,"%y."); next} else
        if(!grepl(pattern = "[m]", x=format))
        {format=paste0(format,"%m."); next} else
          if(!grepl(pattern = "[d]", x=format))
          {format=paste0(format,"%d."); next}
    } 
    stop(paste("unknown date format, check file, column i",i))
  }
  
  format = sub("\\.$", rep="", x = format) #remove tailing "."
  
  # Cumpute date in data
  cont$date=as.POSIXct(strptime(cont[,2], format=format, tz="Etc/GMT-1")) #compute date ("Etc/GMT-1" stands for wintertime (GMT+1)without daylight saving) I dunno why it is "-1" instead of "+1!
  
  
  ### Time-Format
  
  if (any(grepl(cont[,3], pattern = "[mM]"))) #time format using am/pm
    format="%I:%M:%S %p" else
      format="%H:%M:%S" 
  
  tt=difftime(as.POSIXct(strptime(cont[,3], format=format, tz="GMT")), Sys.Date()) #compute time (in minutes)
  cont$date=cont$date+tt #  Add time to the date column
  
  
  
 ### Time-Zone
  
  time_offset = as.numeric(sub(x = names(cont)[grepl(names(cont), pattern = "GMT")], pattern = ".*GMT.(\\d\\d).*", replacement = "\\1"))-1 #extract time offset to GMT+1
  cont$date = cont$date + as.difftime(-time_offset, units = "hours")    #correct dates in case other than "GMT+1" was used
  
  cont <- data.frame(date=cont$date, x=cont$tips) # single file
  
  
###
# Further quality management
# tips due to reading process, calibration, correction errors,...
# also wrong time zone detected for some files; corrected in the following

  # Cabacera1
  if(single_file == "Cabecera1_2011_06_12_Messung.txt"){
    # tips at the end due to reading process
    # dokusheet: "Instrumentenaustausch (Probleme beim Logger Auslesen wegen Sonneneinstrahlung), am 12.06.2011 komplett Fehlwippungen			
    cont <- cont[-c(which(as.character(cont$date) == "2011-06-12 09:16:32"),
                    which(as.character(cont$date) == "2011-06-12 09:16:43"),
                    which(as.character(cont$date) == "2011-06-12 09:16:44"),
                    which(as.character(cont$date) == "2011-06-12 09:16:45"),
                    which(as.character(cont$date) == "2011-06-12 09:16:47"),
                    which(as.character(cont$date) == "2011-06-12 09:19:28"),
                    which(as.character(cont$date) == "2011-06-12 09:19:29"),
                    which(as.character(cont$date) == "2011-06-12 09:53:39"),
                    which(as.character(cont$date) == "2011-06-12 10:22:14"),
                    which(as.character(cont$date) == "2011-06-12 13:15:05"),
                    which(as.character(cont$date) == "2011-06-12 13:15:22"),
                    which(as.character(cont$date) == "2011-06-12 13:15:40"),
                    which(as.character(cont$date) == "2011-06-12 13:15:42"),
                    which(as.character(cont$date) == "2011-06-12 13:15:49"),
                    which(as.character(cont$date) == "2011-06-12 14:11:30"),
                    which(as.character(cont$date) == "2011-06-12 14:11:32")),]
  }
  
  if(single_file == "Cabecera1_2011_06_22_auslesen_corr.txt"){
    #overlap due to wrong manually set start of file
    cont$date[1] = cont$date[1]+3600
  }
  
  if(single_file == "Cabecera1_2011_11_29_a_Auslesen.txt"){
    #first two tips due to reading process
    cont <- cont[-c(which(as.character(cont$date) == "2011-09-28 14:04:40"),
                    which(as.character(cont$date) == "2011-09-28 14:04:42")),]
  }
  
  if(single_file == "Cabecera1_2012_04_19.txt"){
    #last tip due to reading process
    cont <- cont[-c(which(as.character(cont$date) == "2012-04-19 12:52:27")),]
  }
  
  if(single_file == "Cabecera1_2013_04_14.txt"){
    #last tip due to reading process
    cont <- cont[-c(which(as.character(cont$date) == "2013-04-14 08:13:12")),]
  }
  
  if(single_file == "Cabecera1_2013_10_15.txt"){
    #last tip due to reading process
    cont <- cont[-c(which(as.character(cont$date) == "2013-10-15 16:47:09")),]
  }
  
  if(single_file == "Cabecera1_2012_10_17.txt"){
    #first tip due to reading process
    cont <- cont[-c(which(as.character(cont$date) == "2012-04-19 12:56:45")),]
  }
  
  if(single_file == "Cabecera1_2016_04_03.txt"){
    #first tip due to reading process
    cont <- cont[-c(which(as.character(cont$date) == "2015-11-04 16:19:47")),]
  }
  
  if(single_file == "Cabecera1_2015_11_04.txt"){
    #last tips due to reading process
    cont <- cont[-c(which(as.character(cont$date) == "2015-11-04 16:53:20"),
                    which(as.character(cont$date) == "2015-11-04 16:53:24"),
                    which(as.character(cont$date) == "2015-11-04 16:53:51"),
                    which(as.character(cont$date) == "2015-11-04 16:54:03")),]
  }
  
  # Cabacera2
  if(single_file == "Cabecera2_2012_03_28_auslesen.txt"){
    #calibration at the beginning of file
    #first 55 tips due to calibration
    calib = which(cont$date <= "2011-11-16 13:18:10")
    calib = calib[-1]
    cont <- cont[-calib,]
    
    #last two tips due to reading process
    cont <- cont[-c(which(as.character(cont$date) == "2012-03-28 13:04:19"),
                    which(as.character(cont$date) == "2012-03-28 13:04:12")),]
  }
  
  if(single_file == "Cabecera2_2012_04_19.txt"){
      #last tip due to reading process
    cont <- cont[-c(which(as.character(cont$date) == "2012-04-19 13:48:53")),]
  }
  
  if(single_file == "Cabecera2_2013_04_14.txt"){
    #last tip due to reading process
    cont <- cont[-c(which(as.character(cont$date) == "2013-04-14 09:53:40")),]
  }
  
  if(single_file == "Cabecera2_2013_10_15_corr.txt"){
    #dokusheet: "funnel clogged" => corrected
    #no data for time from correction to read out; no like as if no rain => remove read out date
    cont <- cont[-c(which(as.character(cont$date) == "2013-10-15 16:42:00")),]
  }
  
  if(single_file == "Cabecera2_2014_10_14.txt"){
    #first tip due to reading process
    cont <- cont[-c(which(as.character(cont$date) == "2013-10-15 16:45:10")),]
  }
  
  if(single_file == "Cabecera2_2015_11_04_corr.txt"){
    #dokusheet: "Trichet voller Wasser; Daten scheinen bis 23.05.2015 OK" => corrected
    #no data for time from correction to read out; no like as if no rain => replace read out date with last known time +1s
    cont$date[which(as.character(cont$date) == "2015-11-04 19:51:00")] <- cont$date[which(as.character(cont$date) == "2015-11-04 19:51:00")-1]+1
    }
  
  # Villacarli2
  if(single_file == "Villacarli2_2011_12_01_Auslesen_corr.txt"){
    #first tip due to reading/calibration process
    cont$date[1] = cont$date[2]-1 #as added manually exactly same as wrong tip
    cont <- cont[-c(which(as.character(cont$date) == "2011-09-28 15:37:47")),]
  }
  
  if(single_file == "Villacarli2_2013_10_29.txt"){
    #first tip due to reading process
    cont <- cont[-c(which(as.character(cont$date) == "2013-04-14 13:45:12")),]
  }
  
  if(single_file == "Villacarli2_2015_04_30.txt"){
    #last tip due to reading process
    cont <- cont[-c(which(as.character(cont$date) == "2015-04-30 16:56:55")),]
    #wrong time zone in file; GMT+1 is GMT+2; see note dokusheet Ceguera 1:"TB noch auf GMT+1[...] Toughbook trotz GMT+1 noch auf 16:36 (vermutlich per Hand auf Sommerzeit aktualisiert, obwohl autom. Umstellung deaktiviert war)"
    cont$date <- cont$date-3600
  }
  
  if(single_file == "Villacarli2_2016_04_03.txt"){
    #first tip due to reading process
    cont <- cont[-c(which(as.character(cont$date) == "2015-11-04 10:50:07")),]
  }
  
  if(single_file == "Villacarli2_2012_03_27_auslesen.txt"){
    #first two tips due to reading process
    cont <- cont[-c(which(as.character(cont$date) == "2012-03-27 14:00:39"),
                    which(as.character(cont$date) == "2012-03-27 14:00:42")),]
  }
  
  # Ceguera1
  
  if(single_file == "Ceguera1_2011_06_15_auslesen.txt"){
    #calibration on 13.06.2011 + tips due to reading process on 15.06 (see dokusheet) => remove all after start calibration
    cont <- cont[-(which(as.character(cont$date) == "2011-06-13 13:16:32") : which(as.character(cont$date) == "2011-06-15 15:08:31")),]
    
  }
  
  if(single_file == "Ceguera1_2011_12_01_Auslesen_corr.txt" ){
    #calibration at the beginning of file; remove calibration
    cont <- cont[-(which(as.character(cont$date) == "2011-09-26 14:38:00") : which(as.character(cont$date) == "2011-09-26 15:03:19")),]
  }
  
  if(single_file == "Ceguera1_2012_03_23_auslesen_corr.txt"){
    #logger did not record! dokusheet: "Logger war gestoppt seit letztem Auslesen"
    #no data, not no rainfall
    cont <- cont[-c(which(as.character(cont$date) == "2011-12-01 15:19:00"),which(as.character(cont$date) == "2012-03-23 15:25:00")),]
  }
  
  if(single_file == "Ceguera1_2012_10_17.txt"){
    #last tip due to reading process
    cont <- cont[-c(which(as.character(cont$date) == "2012-10-17 13:45:13"),
                    which(as.character(cont$date) == "2012-10-17 13:45:15")),]
  }
  
  if(single_file == "Ceguera1_2013_04_15.txt"){
    #last and first tip due to reading process
    cont <- cont[-c(which(as.character(cont$date) == "2013-04-03 13:36:20"),
                    which(as.character(cont$date) == "2013-04-15 08:29:08")),]
  }
  
  if(single_file == "Ceguera1_2013_10_29.txt"){
    #first tip due to reading process
    cont <- cont[-c(which(as.character(cont$date) == "2013-04-15 08:41:28")),]
  }
  
  if(single_file == "Ceguera1_2014_10_26.txt"){
    #device defect, see dokusheet; remove read out date
    #no data for time from defect to read out; now like as if no rain => remove read out date
    cont <- cont[-c(which(as.character(cont$date) == "2014-10-26 19:04:00")),]
  }
  
  if(single_file == "Ceguera1_2015_11_06.txt"){
    #wrong time zone in file; GMT+1 is GMT+2; see note dokusheet:"TB noch auf GMT+1[...] Toughbook trotz GMT+1 noch auf 16:36 (vermutlich per Hand auf Sommerzeit aktualisiert, obwohl autom. Umstellung deaktiviert war)"
    cont$date <- cont$date-3600
  }
  
  # Badland/Villacarli1
  
  if(single_file == "Badland_2011_09_16_auslesen.txt"){
    #calibration at the beginning of file; remove calibration
    cont <- cont[-(which(as.character(cont$date) == "2011-06-22 13:02:22") : which(as.character(cont$date) == "2011-06-22 13:24:52")),]
  }
  
  if(single_file == "Badland_2012_03_27_auslesen_corr.txt"){
    #last tip due to reading process
    cont <- cont[-c(which(as.character(cont$date) == "2012-03-27 15:04:23")),]
  }
  
  if(single_file == "Badland_2013_04_14.txt"){
    #first five tips due to reading process
    cont <- cont[-(which(as.character(cont$date) == "2013-04-03 10:04:44") : which(as.character(cont$date) == "2013-04-03 10:05:49")),]
  }
  
  if(single_file == "Badland_2013_10_18.txt"){
    #last two tips due to reading process
    cont <- cont[-(which(as.character(cont$date) == "2013-10-18 16:56:09") : which(as.character(cont$date) == "2013-10-18 16:56:16")),]
  }
  
  if(single_file == "Badland_2014_10_17.txt"){
    #last tip due to reading process
    cont <- cont[-c(which(as.character(cont$date) == "2014-10-17 08:26:01")),]
  }
  
  if(single_file == "Badland_2015_11_04_corr.txt"){
    #end corrected; removed 'Loggertrichter Verstopft...'; 
    cont <- cont[-c(which(as.character(cont$date) == "2015-11-04 10:34:00")),]
  }
  
  if(single_file == "Badland_2013_10_28_Event.txt"){
    #wrong time zone in file; GMT+1 is GMT+2; see notes sesam
    cont$date <- cont$date-3600
  }
  
  if(single_file == "Badland_2015_04_30.txt"){
    #wrong time zone in file; GMT+1 is GMT+2; see note dokusheet Ceguera1:"TB noch auf GMT+1[...] Toughbook trotz GMT+1 noch auf 16:36 (vermutlich per Hand auf Sommerzeit aktualisiert, obwohl autom. Umstellung deaktiviert war)"
    cont$date <- cont$date-3600
  }
  
  
  #Carrasquero2
  
  if(single_file == "Carrasquero2_2011_10_10_corr_obsolete.txt"){
    #calibration at the beginning of file; remove calibration
    cont <- cont[-(which(as.character(cont$date) == "2011-06-11 13:51:14") : which(as.character(cont$date) == "2011-06-11 14:43:30")),]
  }
  
  if(single_file == "Carrasquero2_2013_04_15.txt"){
    #last tip due to reading process
    cont <- cont[-c(which(as.character(cont$date) == "2013-04-15 14:18:31")),]
  }
  
  if(single_file == "Carrasquero2_2013_10_29_corr.txt"){
    #beginning corrected; NA same time as first tip
    cont$date[1] = cont$date[1]-1
  }
  
  if(single_file == "Carrasquero2_2014_10_19_corr.txt"){
    #end corrected: "Dreck und Wasser im Trichter"; remove read out date; no data, not no rain
    cont <- cont[-c(which(as.character(cont$date) == "2014-10-19 12:21:00")),]
  }
  
  if(single_file == "Carrasquero2_2015_11_05_corr.txt"){
    #end corrected; remove read out date; no data, not no rain
    cont <- cont[-c(which(as.character(cont$date) == "2015-11-05 09:43:00")),]
    cont$date[length(cont$date)] = cont$date[which(as.character(cont$date) == "2015-04-20 11:24:21")]+1
    cont <- cont[-c(which(as.character(cont$date) == "2015-04-20 11:25:00")),]
    #wrong time zone in file; GMT+1 is GMT+2; see note dokusheet Ceguera1
    cont$date <- cont$date-3600
  }
  
  if(single_file == "Carrasquero2_2012_10_11.txt"){
    # in data change dat format 12 and 2012 => 2020 in data; substract 4 years
    cont$date[which(as.numeric(format(cont$date,"%Y")) ==2020)] = cont$date[which(as.numeric(format(cont$date,"%Y")) ==2020)] - (60*60*24*365*6 + 60*60*24*366*2)
    }

  #Ceguera2
  
  if(single_file == "Ceguera2_2011_12_05_corr.txt"){
    #end corrected: remove manually added read out date; no data, not no rain
    #beginning corrected; NA same time as first tip
    cont <- cont[-c(which(as.character(cont$date) == "2011-12-05 16:00:00")),]
    cont$date[1] = cont$date[1]-1
  }
  
  if(single_file == "Ceguera2_2012_03_28_auslesen.txt"){
    #last tip due to reading process
    cont <- cont[-c(which(as.character(cont$date) == "2012-03-28 08:33:24")),]
  }
  
  if(single_file == "Ceguera2_2012_10_17.txt"){
    #first tip due to reading process
    cont <- cont[-c(which(as.character(cont$date) == "2012-04-22 12:49:53")),]
  }
  
  if(single_file == "Ceguera2_2013_04_15.txt"){
    #first two tips due to reading process
    cont <- cont[-c(which(as.character(cont$date) == "2013-04-03 16:04:06"),
                    which(as.character(cont$date) == "2013-04-03 16:04:10")),]
  }
  
  if(single_file == "Ceguera2_2015_11_05.txt"){
    #wrong time zone in file; GMT+1 is GMT+2; see note dokusheet Ceguera1
    cont$date <- cont$date-3600
  }
  
  if(single_file == "Ceguera2_2013_10_29_corr.txt"){
    #device damaged; see dokusheet "data seems to be corrupted after 14.7.2013, 13:00"
    #remove end
    cont <- cont[-(which(as.character(cont$date) == "2013-09-07 12:09:08") : which(as.character(cont$date) == "2013-10-29 14:17:46")),]
    cont$date[nrow(cont)] = cont$date[nrow(cont)-1]+1
    }
  
  
  #Lascuarre1
  
  if(single_file == "Lascuarre1_2013_04_15_corr.txt"){
    #first tip due to reading process
    cont <- cont[-c(which(as.character(cont$date) == "2013-04-04 12:37:11")),]
  }
  
  if(single_file == "Lascuarre1_2013_04_04_corr.txt"){
    #last date added manually; does not represent real observation coverage
    cont$date[nrow(cont)] = cont$date[nrow(cont)-1]+1
  }
  
  if(single_file == "Lascuarre1_2015_11_03.txt"){
    #first tip due to reading process
    cont <- cont[-c(which(as.character(cont$date) == "2015-04-29 15:36:24")),]
  }
  
  if(single_file == "Lascuarre1_2015_04_29_es.txt"){
    #wrong time zone in file; GMT+1 is GMT+2; see note dokusheet Ceguera1
    cont$date <- cont$date-3600
  }
  
  
  #Lacuarre2
  
  if(single_file == "Lascuarre2_2012_03_29_auslesen.txt"){
    #first two tips due to reading process
    cont <- cont[-c(which(as.character(cont$date) == "2012-03-29 08:32:33"),
                    which(as.character(cont$date) == "2012-03-29 08:32:36"),
                    which(as.character(cont$date) == "2012-03-29 08:32:38")),]
  }
  
  if(single_file == "Lascuarre2_2013_04_15.txt"){
    #first tip due to reading process
    cont <- cont[-c(which(as.character(cont$date) == "2013-03-25 16:16:01")),]
  }
  
  if(single_file == "Lascuarre2_2013_11_15_es.txt"){
    #first tip due to reading process
    cont <- cont[-c(which(as.character(cont$date) == "2013-04-15 09:47:02")),]
  }
  
  if(single_file == "Lascuarre2_2014_10_28.txt"){
    #first tip due to reading process
    cont <- cont[-c(which(as.character(cont$date) == "2013-11-15 17:26:58")),]
  }
  
  if(single_file == "Lascuarre2_2015_04_29_es.txt"){
    #last two tips due to reading process
    cont <- cont[-c(which(as.character(cont$date) == "2015-04-29 16:46:02"),
                    which(as.character(cont$date) == "2015-04-29 16:46:23")),]
    #wrong time zone in file; GMT+1 is GMT+2; see note dokusheet Ceguera1
    cont$date <- cont$date-3600
  }
  
  if(single_file == "Lascuarre2_2015_11_03_corr.txt"){
    #no rain recorded; just two NAs in file to mark start and end; see dokusheet
    cont <- cont[-c(which(as.character(cont$date) == "2015-04-29 15:49:42"),which(as.character(cont$date) == "2015-11-03 16:09:00")),]
  }
  
  #Lascuarra3
  
  if(single_file == "Lascuarre3_2013_04_04.txt"){
    #first tip due to reading process
    cont <- cont[-c(which(as.character(cont$date) == "2013-03-27 13:06:26")),]
  }
  
  if(single_file == "Lascuarre3_2015_04_29_es.txt"){
    #first tip due to reading process
    #additional NA at the end due to reading process
    cont <- cont[-c(which(as.character(cont$date) == "2014-10-18 15:03:16"),
                    length(cont$date)),]
    #wrong time zone in file; GMT+1 is GMT+2; see note dokusheet Ceguera1
    cont$date <- cont$date-3600
  }
  
  if(single_file == "Lascuarre3_2014_10_18.txt"){
    #device damaged; see dokusheet; station comparison reveals device not functioning correctly since 28.05.2014
    #remove end
    cont <- cont[-(which(as.character(cont$date) == "2014-05-28 00:28:20") : which(as.character(cont$date) == "2014-10-18 13:18:44")),]
    cont$date[nrow(cont)] = cont$date[nrow(cont)-1]+1
  }
 
###

  cont_collected <-  rbind(cont_collected, cont) # collect all single files from loop
  
  #Aggregated data (continous data for single files, hours)
  
  cont$date_full_hour =as.POSIXct(trunc(cont$date, unit="hour"))
  cont$x_new = cont$x
  cont$x_new[which(is.na(cont$x_new))] = 0 #NAs beginning and ending to 0
  na_sum=function(x){ifelse(all(is.na(x)), NA, sum(x, na.rm=TRUE))}  #sum up disregarding NAs, except when all values are NA
 
   if(nrow(cont) > 0) {
  cont_aggr <-  aggregate(x = cont$x_new, by=list(date=cont$date_full_hour), FUN=na_sum) 
  cont$date_full_hour = NULL
  cont$x_new =NULL
  #cont_aggr <-  cont_aggr[-which(is.na(cont_aggr$x)),] # remove NA; original NA for end and beginnig 
  
  #cont_aggr$date=as.POSIXct(strptime(cont_aggr$date, format="%Y-%m-%d %H", tz="Etc/GMT-1"))
  
  start_date = as.POSIXct(strptime(as.character(min(cont_aggr$date, na.rm=T)), "%Y-%m-%d %H:%M:%S", tz="etc/GMT-1"))  # nur WZ
  end_date = as.POSIXct(strptime(as.character(max(cont_aggr$date, na.rm=T)), "%Y-%m-%d %H:%M:%S", tz="etc/GMT-1"))  # nur WZ
  full_time = seq(from = start_date , to=end_date, by='hour')
  
  full_data = data.frame(date=full_time, x=with(cont_aggr, x[match(full_time, date)]))
  full_data$x[which(is.na(full_data$x))] = 0} else {full_data <- cont}
  
  cont_collected_aggr = rbind(cont_collected_aggr , full_data)
  
  ### For a first queck after the run
  date_min <- min(c(cont$date, date_min), na.rm=TRUE)
  date_max <- max(c(cont$date, date_max), na.rm=TRUE)
  p_total  <- p_total + sum(cont$x, na.rm=TRUE)
  
  n <- n+1
  
  if(n == length(file_list)) # all files processed
  {
  
  #Save as .RData
  save(file = paste0("MergeData/",station_id_golm,"_merged.RData"), list="cont_collected")  
  save(file = paste0("MergeData/",station_id_golm,"_merged_aggr.RData"), list="cont_collected_aggr")  
    
###
  #Plots
  
  #aggregate tips to full hour for plot
  cont_collected$date_full_hour =as.POSIXct(trunc(cont_collected$date, unit="hour"))
  na_sum=function(x){ifelse(all(is.na(x)), NA, sum(x, na.rm=TRUE))}  #sum up disregarding NAs, except when all values are NA
  cont_aggr <-  aggregate(x = cont_collected$x, by=list(date=cont_collected$date_full_hour), FUN=na_sum)
  cont_collected$date_full_hour = NULL
  cont_aggr <-  cont_aggr[-which(is.na(cont_aggr$x)),] # remove NA; original NA for end and beginnig added next lines again
  
  start_end_files_NA = cont_collected[,1:2][which(is.na(cont_collected$x)),]
  cont_aggr <- rbind(start_end_files_NA,cont_aggr)
  
  sorted_ix=sort.int(cont_aggr$date, index.return = TRUE)$ix
  cont_aggr = cont_aggr[sorted_ix,]
  
  #Remove NA to be able to make cunsum-plot
  cont_cunsum <- cont_collected[-which(is.na(cont_collected$x)),]
  
  sub2011 <- subset(cont_aggr, format(date, "%Y")=="2011")
  sub2012 <- subset(cont_aggr, format(date, "%Y")=="2012")
  sub2013 <- subset(cont_aggr, format(date, "%Y")=="2013")
  sub2014 <- subset(cont_aggr, format(date, "%Y")=="2014")
  sub2015 <- subset(cont_aggr, format(date, "%Y")=="2015")
  sub2016 <- subset(cont_aggr, format(date, "%Y")=="2016")
  filename <- station_prefix
  
  pdf(paste("Plots/",filename,".pdf",sep = ""),width=10,height=8)
  
  if(nrow(cont_aggr) >0){
    plot(cont_aggr$date, cont_aggr$x, main=paste(filename), ylab = "Precipitation [mm/h]", xlab= "Time", type="h")
    #abline(v=as.numeric(cont_aggr$date[which(is.na(cont_aggr$x))]), lwd=1, col='red')
    legend("topleft", c("Range", as.character(round(range(cont_aggr$x, na.rm=T), digits = 1))))}
  
  cont_cunsum <- cont_collected[-which(is.na(cont_collected$x)),]
  plot(cont_cunsum$date, cumsum(cont_cunsum$x), pch=".",xlab="Time",ylab="Cumulative Sum")
  abline(v=as.numeric(cont_collected$date[which(is.na(cont_collected$x))]), lwd=1, col='red')
  
  if(nrow(sub2011) >0){
    plot(sub2011$date, sub2011$x, main=paste(filename,"_2011",sep=""), ylab = "Precipitation [mm/h]", xlab= "Time", type="h")
    #abline(v=as.numeric(sub2011$date[which(is.na(sub2011$x))]), lwd=1, col='red')
    legend("topleft", c("Range", as.character(round(range(sub2011$x, na.rm=T), digits = 1))))}
  
  if(nrow(sub2012) >0){
    plot(sub2012$date, sub2012$x, main=paste(filename,"_2012",sep=""), ylab = "Precipitation [mm/h]", xlab= "Time", type="h")
    legend("topleft", c("Range", as.character(round(range(sub2012$x, na.rm=T), digits = 1))))}
  
  if(nrow(sub2013) >0){
    plot(sub2013$date, sub2013$x, main=paste(filename,"_2013",sep=""), ylab = "Precipitation [mm/h]", xlab= "Time", type="h")
    legend("topleft", c("Range", as.character(round(range(sub2013$x, na.rm=T), digits = 1))))}
  
  if(nrow(sub2014) >0){
    plot(sub2014$date, sub2014$x, main=paste(filename,"_2014",sep=""), ylab = "Precipitation [mm/h]", xlab= "Time", type="h")
    legend("topleft", c("Range", as.character(round(range(sub2014$x, na.rm=T), digits = 1))))}
  
  if(nrow(sub2015) >0){
    plot(sub2015$date, sub2015$x, main=paste(filename,"_2015",sep=""), ylab = "Precipitation [mm/h]", xlab= "Time", type="h")
    legend("topleft", c("Range", as.character(round(range(sub2015$x, na.rm=T), digits = 1))))}
  
  if(nrow(sub2016) >0){
    plot(sub2016$date, sub2016$x, main=paste(filename,"_2016",sep=""), ylab = "Precipitation [mm/h]", xlab= "Time", type="h")
    legend("topleft", c("Range", as.character(round(range(sub2016$x, na.rm=T), digits = 1))))}
  
  dev.off()
  
  }
  
  
}


###
#first quality check after the run

print(paste(date_min,"-",date_max,":",p_total))
summary(cont_collected)
length(file_list)
which(cont_collected$x > 0.2020)
cont_collected[which(is.na(cont_collected$x)),]


### Step 2: Station comparison ####

load("MergeData/Cab1_merged.RData") ; Cab1 <- cont_collected
load("MergeData/Cab2_merged.RData") ; Cab2 <- cont_collected
load("MergeData/Car_merged.RData")  ; Car  <- cont_collected
load("MergeData/Ceg1_merged.RData") ; Ceg1 <- cont_collected
load("MergeData/Ceg2_merged.RData") ; Ceg2 <- cont_collected
load("MergeData/Las1_merged.RData") ; Las1 <- cont_collected
load("MergeData/Las2_merged.RData") ; Las2 <- cont_collected
load("MergeData/Lascuarre_tube_merged.RData") ; Lascuarre_tube <- cont_collected
load("MergeData/Via1_merged.RData") ; Via1 <- cont_collected
load("MergeData/Via2_merged.RData") ; Via2 <- cont_collected

load("MergeData/Cab1_merged_aggr.RData") ; Cab1_aggr <- cont_collected_aggr
load("MergeData/Cab2_merged_aggr.RData") ; Cab2_aggr <- cont_collected_aggr
load("MergeData/Car_merged_aggr.RData")  ; Car_aggr  <- cont_collected_aggr
load("MergeData/Ceg1_merged_aggr.RData") ; Ceg1_aggr <- cont_collected_aggr
load("MergeData/Ceg2_merged_aggr.RData") ; Ceg2_aggr <- cont_collected_aggr
load("MergeData/Las1_merged_aggr.RData") ; Las1_aggr <- cont_collected_aggr
load("MergeData/Las2_merged_aggr.RData") ; Las2_aggr <- cont_collected_aggr
load("MergeData/Lascuarre_tube_merged_aggr.RData") ; Lascuarre_tube_aggr <- cont_collected_aggr
load("MergeData/Via1_merged_aggr.RData") ; Via1_aggr <- cont_collected_aggr
load("MergeData/Via2_merged_aggr.RData") ; Via2_aggr <- cont_collected_aggr

station_compare <- function(data_1, data_2, data_3, data_4){
  
  pdf(paste("Plots/","station_compare.pdf",sep = ""),width=10,height=8)
  
  # Cumsum-Plot station 1
  data_1NA <- data_1[-which(is.na(data_1$x)),]
  if(nrow(data_1NA) > 0){
  plot(data_1NA$date, cumsum(data_1NA$x), pch=".", main="Station 1",xlab="Time", ylab="Cumulative Sum")
  abline(v=as.numeric(data_1$date[which(is.na(data_1$x))]), lwd=1, col='red')}
  
  # Cumsum-Plot station 2
  data_2NA <- data_2[-which(is.na(data_2$x)),]
  if(nrow(data_2NA) > 0){
  plot(data_2NA$date, cumsum(data_2NA$x), pch=".", main="Station 2", xlab="Time", ylab="Cumulative Sum")
  abline(v=as.numeric(data_2$date[which(is.na(data_2$x))]), lwd=1, col='red')}
  
  # Individual cumsum in one plot
  if(nrow(data_2NA) > 0){
  plot(data_1NA$date, cumsum(data_1NA$x), pch=".", ylim=c(0,max(c(cumsum(data_1NA$x),cumsum(data_2NA$x)))),main="Comparison cumulative sum", xlab="Time", ylab="Cumulative Sum")
  points(data_2NA$date, cumsum(data_2NA$x), pch=".",col=2)
  grid(5,5)
  legend("topleft",c("Station 1","Station 2"), pch=19, col=1:2)}
  
  # Cumsum-Comparison-Plot without prior aggregation
  merge_data = merge(data_1, data_2, by = "date", all = TRUE) 
  merge_data$x.x[which(is.na(merge_data$x.x))] = 0 # for comparative cunsum plot NAs to zero
  merge_data$x.y[which(is.na(merge_data$x.y))] = 0
 
  plot(cumsum(merge_data$x.x), cumsum(merge_data$x.y), pch=".",  main= "Station comparison using cumulative sum",sub="data not aggregated; merge (... all=TRUE); NA = 0", xlab="Station 1", ylab="Station 2")
  grid(5,5)
  
  # Cumsum-Comparison-Plot with prior aggregation minute; merge all = False
  data_1$date_full_min =as.POSIXct(trunc(data_1$date, unit="min"))
  na_sum=function(x){ifelse(all(is.na(x)), NA, sum(x, na.rm=TRUE))}  #sum up disregarding NAs, except when all values are NA
  data_aggr1 <-  aggregate(x = data_1$x, by=list(date=data_1$date_full_min), FUN=na_sum)
  data_1$date_full_min = NULL
  data_aggr1 <-  data_aggr1[-which(is.na(data_aggr1$x)),] # remove NA; original NA for end and beginnig added next lines again
  
  data_2$date_full_min =as.POSIXct(trunc(data_2$date, unit="min"))
  na_sum=function(x){ifelse(all(is.na(x)), NA, sum(x, na.rm=TRUE))}  #sum up disregarding NAs, except when all values are NA
  data_aggr2 <-  aggregate(x = data_2$x, by=list(date=data_2$date_full_min), FUN=na_sum)
  data_2$date_full_min = NULL
  data_aggr2 <-  data_aggr2[-which(is.na(data_aggr2$x)),] # remove NA; original NA for end and beginnig added next lines again
  
  merge_data_aggr_min = merge(data_aggr1, data_aggr2, by = "date", all = FALSE) 
  
  if(nrow(merge_data_aggr_min) > 0){
  plot(cumsum(merge_data_aggr_min$x.x), cumsum(merge_data_aggr_min$x.y), pch=".", main= "Station comparison using cumulative sum",sub="prior data aggregated to minutes; merge (... all=FALSE)", xlab="Station 1", ylab="Station 2")
  grid(5,5)
  abline(0,1, col="orange")}
  
  # Cumsum-Comparison-Plot with aggregation 15 min
  start_date = as.POSIXct(strptime("2011-01-01 00:00", "%Y-%m-%d %H:%M", tz="etc/GMT-1"))  # nur WZ
  end_date = as.POSIXct(strptime("2016-12-31 23:59", "%Y-%m-%d %H:%M", tz="etc/GMT-1"))  # nur WZ
  full_time = seq(from = start_date , to=end_date, by='1 min')
  
  full_data_1 = data.frame(date=full_time, x=with(data_aggr1, x[match(full_time, date)]))
  full_data_2 = data.frame(date=full_time, x=with(data_aggr2, x[match(full_time, date)]))
  full_data_1$x[which(is.na(full_data_1$x))] = 0
  full_data_2$x[which(is.na(full_data_2$x))] = 0
  
  brk = "15 min"
  
  full_data_15min_1 <- transform(full_data_1, date = cut(as.POSIXct(full_data_1$date, format = "%Y-%m-%d %H:%M:%S"), breaks = brk))
  full_data_15min_2 <- transform(full_data_2, date = cut(as.POSIXct(full_data_2$date, format = "%Y-%m-%d %H:%M:%S"), breaks = brk))
  
  full_data_aggr_15min_1 <- aggregate(full_data_15min_1$x~date, full_data_15min_1, sum)
  full_data_aggr_15min_2 <- aggregate(full_data_15min_2$x~date, full_data_15min_2, sum)
 
  data_15min_1 <- full_data_aggr_15min_1[which(full_data_aggr_15min_1$`full_data_15min_1$x`!=0),]
  data_15min_2 <- full_data_aggr_15min_2[which(full_data_aggr_15min_2$`full_data_15min_2$x`!=0),]
  
  
  merge_data_15min = merge(data_15min_1, data_15min_2, by = "date", all = FALSE) 
  colnames(merge_data_15min) = c("date", "x1","x2")
  
  if(nrow(merge_data_15min) > 1){
  plot(cumsum(merge_data_15min$x1), cumsum(merge_data_15min$x2), pch=".", main= "Station comparison using cumulative sum",sub="prior data aggregated to 15 minutes; merge (... all=FALSE)", xlab="Station 1", ylab="Station 2")
  grid(5,5)
  abline(0,1, col="orange")}
  
  # Cumsum-Comparison-Plot with prior aggragation hour; merge all = False
  data_1$date_full_hour =as.POSIXct(trunc(data_1$date, unit="hour"))
  na_sum=function(x){ifelse(all(is.na(x)), NA, sum(x, na.rm=TRUE))}  #sum up disregarding NAs, except when all values are NA
  data_aggr1 <-  aggregate(x = data_1$x, by=list(date=data_1$date_full_hour), FUN=na_sum)
  data_1$date_full_hour = NULL
  data_aggr1 <-  data_aggr1[-which(is.na(data_aggr1$x)),] # remove NA; original NA for end and beginnig added next lines again
  
  data_2$date_full_hour =as.POSIXct(trunc(data_2$date, unit="hour"))
  na_sum=function(x){ifelse(all(is.na(x)), NA, sum(x, na.rm=TRUE))}  #sum up disregarding NAs, except when all values are NA
  data_aggr2 <-  aggregate(x = data_2$x, by=list(date=data_2$date_full_hour), FUN=na_sum)
  data_2$date_full_hour = NULL
  data_aggr2 <-  data_aggr2[-which(is.na(data_aggr2$x)),] # remove NA; original NA for end and beginnig added next lines again
  
  merge_data_aggr_hour = merge(data_aggr1, data_aggr2, by = "date", all = FALSE) 
  
  plot(cumsum(merge_data_aggr_hour$x.x), cumsum(merge_data_aggr_hour$x.y), pch=".", main= "Station comparison using cumulative sum",sub="prior data aggregated to hourly values; merge (... all=FALSE)", xlab="Station 1", ylab="Station 2")
  grid(5,5)
  abline(0,1, col="orange") 
  
  # Cumsum-Comparison-Plot with prior aggragation day; merge all = False
  data_1$date_day =as.POSIXct(trunc(data_1$date, unit="day"))
  na_sum=function(x){ifelse(all(is.na(x)), NA, sum(x, na.rm=TRUE))}  #sum up disregarding NAs, except when all values are NA
  data_aggr1 <-  aggregate(x = data_1$x, by=list(date=data_1$date_day), FUN=na_sum)
  data_1$date_day = NULL
  data_aggr1 <-  data_aggr1[-which(is.na(data_aggr1$x)),] # remove NA; original NA for end and beginnig added next lines again
  
  data_2$date_day =as.POSIXct(trunc(data_2$date, unit="day"))
  na_sum=function(x){ifelse(all(is.na(x)), NA, sum(x, na.rm=TRUE))}  #sum up disregarding NAs, except when all values are NA
  data_aggr2 <-  aggregate(x = data_2$x, by=list(date=data_2$date_day), FUN=na_sum)
  data_2$date_day = NULL
  data_aggr2 <-  data_aggr2[-which(is.na(data_aggr2$x)),] # remove NA; original NA for end and beginnig added next lines again
  
  merge_data_aggr_day = merge(data_aggr1, data_aggr2, by = "date", all = FALSE) 
  
  if(nrow(merge_data_aggr_day) > 0){
  plot(cumsum(merge_data_aggr_day$x.x), cumsum(merge_data_aggr_day$x.y), pch=20, main= "Station comparison using cumulative sum",sub="prior data aggregated to daily values; merge (... all=FALSE)", xlab="Station 1", ylab="Station 2")
  grid(5,5)
  abline(0,1, col="orange")} 
  
  # Cumsum-Plot station 1
  plot(data_3$date, cumsum(data_3$x), pch=".", main="Station 1",sub="single files aggregated hourly, continous times series",xlab="Time", ylab="Cumulative Sum")
  
  # Cumsum-Plot station 1
  plot(data_4$date, cumsum(data_4$x), pch=".", main="Station 2",sub="single files aggregated hourly, continous times series",xlab="Time", ylab="Cumulative Sum")
  
  # Cumsum-Comparison-Plot hourly continous data
  merge_data = merge(data_3, data_4, by = "date", all = FALSE) 
  merge_data$x.x[which(is.na(merge_data$x.x))] = 0 # for comparative cunsum plot NAs to zero
  merge_data$x.y[which(is.na(merge_data$x.y))] = 0
  
  plot(cumsum(merge_data$x.x), cumsum(merge_data$x.y), pch=".",  main= "Station comparison using cumulative sum",sub="single files aggregated hourly,continous times series; merge (... all=FALSE);", xlab="Station 1", ylab="Station 2")
  grid(5,5)
  abline(0,1, col="orange") 
  
  # Individual stations hourly continours data, merge()
  plot(merge_data$date, cumsum(merge_data$x.x), pch=".", ylim=c(0,max(c(cumsum(merge_data$x.x),cumsum(merge_data$x.y)))),main="Station comparison",sub="single files aggregated hourly, continous times series; merge (... all=FALSE)",xlab="Time", ylab="Cumulative Sum")
  points(merge_data$date, cumsum(merge_data$x.y), pch=".", col="red")
  points(merge_data$date, abs((cumsum(merge_data$x.x)-cumsum(merge_data$x.y))), pch=".", col="blue")
  legend("topleft",c("Station 1","Station 2","|Station 1 - Station 2|"), pch=19, col=c("black","red","blue"))
  
  
    dev.off()
}

station_compare(Via1,Via2,Via1_aggr,Via2_aggr) # data_1/2 tipping point data; data_3/4 aggregated data


### Step 3: CUASHI data ####

station_id_golm="Cab1"
load("MergeData/Cab1_merged.RData")

station_id_golm="Cab2"
load("MergeData/Cab2_merged.RData")

station_id_golm="Via2"
load("MergeData/Via2_merged.RData")

station_id_golm="Ceg2"
load("MergeData/Ceg2_merged.RData")

station_id_golm="Car"
load("MergeData/Car_merged.RData")

station_id_golm="Las1"
load("MergeData/Las1_merged.RData")

station_id_golm="Las2"
load("MergeData/Las2_merged.RData")

station_id_golm="Via1"
load("MergeData/Via1_merged.RData")

station_id_golm="Ceg1"
load("MergeData/Ceg1_merged.RData")

station_id_golm="Lascuarre_tube"
load("MergeData/Lascuarre_tube_merged.RData")


n= nrow(cont_collected)

dtutc<- format(cont_collected$date, tz="UTC")
dtloc<- format(cont_collected$date, tz="Etc/GMT-1") # LocalDateTime winter time GMT+1
utc_offset = as.numeric(format(cont_collected$date, format = "%z"))/100
sitecode <- rep(station_id_golm,n)
varcode <- rep("rainfall_breakpoint",n)
censorc	<- rep("nc", n) 
methodc	<- rep("tipping_bucket_Davis", n)
#methodc	<- rep("tipping_bucket_Thies", n) # for Las3, Lascuarre tube, different system
sourcec <- rep("SESAMII_meteo", n)
qualityc <- rep("1",n)

output_table=data.frame(DataValue=round(cont_collected$x, digits = 3),
                        ValueAccuracy="",
                        LocalDateTime=dtloc,
                        UTCOffset=utc_offset,
                        DateTimeUTC=dtutc,
                        SiteCode=sitecode,
                        VariableCode=varcode,
                        OffsetValue="",
                        OffsetTypeCode="",
                        CensorCode = censorc,
                        QualifierCode="",
                        MethodCode = methodc,
                        SourceCode = sourcec,
                        SampleCode="",
                        DerivedFromID="",
                        QualityControlLevelCode= qualityc )

write.table(file=paste0("CuashiData/",station_id_golm, "_precip.csv"),x = output_table, na = "-9999", sep=",", row.names = FALSE, quote = FALSE)


### Step 4: Final Check ####

#Plots

final <- function(filename){
  
cuashi_file = paste("CuashiData/", filename, sep="")
cuashi <- read.csv(cuashi_file, header=T, na.strings="-9999", sep=",", dec="." )

cuashi$LocalDateTime = as.POSIXct(strptime(cuashi$LocalDateTime, format="%Y-%m-%d %H:%M:%S", tz="etc/GMT-1"))


#aggregate tips to full hour for plot
cuashi$full_hour =as.POSIXct(trunc(cuashi$LocalDateTime, unit="hour"))

na_sum=function(x){ifelse(all(is.na(x)), NA, sum(x, na.rm=TRUE))}  #sum up disregarding NAs, except when all values are NA
cuashi_aggr <-  aggregate(x = cuashi$DataValue, by=list(date=cuashi$full_hour), FUN=na_sum)

sub2011 <- subset(cuashi_aggr, format(date, "%Y")=="2011")
sub2012 <- subset(cuashi_aggr, format(date, "%Y")=="2012")
sub2013 <- subset(cuashi_aggr, format(date, "%Y")=="2013")
sub2014 <- subset(cuashi_aggr, format(date, "%Y")=="2014")
sub2015 <- subset(cuashi_aggr, format(date, "%Y")=="2015")
sub2016 <- subset(cuashi_aggr, format(date, "%Y")=="2016")
filename <- cuashi$SiteCode[1]

pdf(paste("Plots/",filename,"_Cuashi.pdf",sep = ""),width=10,height=8)

if(nrow(cuashi_aggr) >0){
  plot(cuashi_aggr$date, cuashi_aggr$x, main=paste(filename), ylab = "Precipitation [mm/h]", xlab= "Time", type="h")
  #abline(v=as.numeric(cont_aggr$date[which(is.na(cont_aggr$x))]), lwd=1, col='red')
  legend("topleft", c("Range", as.character(round(range(cuashi_aggr$x, na.rm=T), digits = 1))))}

cont_cunsum <- cuashi[-which(is.na(cuashi$DataValue)),]
plot(cont_cunsum$LocalDateTime, cumsum(cont_cunsum$DataValue), pch=".",xlab="Time",ylab="Cumulative Sum")

if(nrow(sub2011) >0){
  plot(sub2011$date, sub2011$x, main=paste(filename,"_2011",sep=""), ylab = "Precipitation [mm/h]", xlab= "Time", type="h")
  #abline(v=as.numeric(sub2011$date[which(is.na(sub2011$x))]), lwd=1, col='red')
  legend("topleft", c("Range", as.character(round(range(sub2011$x, na.rm=T), digits = 1))))}

if(nrow(sub2012) >0){
  plot(sub2012$date, sub2012$x, main=paste(filename,"_2012",sep=""), ylab = "Precipitation [mm/h]", xlab= "Time", type="h")
  legend("topleft", c("Range", as.character(round(range(sub2012$x, na.rm=T), digits = 1))))}

if(nrow(sub2013) >0){
  plot(sub2013$date, sub2013$x, main=paste(filename,"_2013",sep=""), ylab = "Precipitation [mm/h]", xlab= "Time", type="h")
  legend("topleft", c("Range", as.character(round(range(sub2013$x, na.rm=T), digits = 1))))}

if(nrow(sub2014) >0){
  plot(sub2014$date, sub2014$x, main=paste(filename,"_2014",sep=""), ylab = "Precipitation [mm/h]", xlab= "Time", type="h")
  legend("topleft", c("Range", as.character(round(range(sub2014$x, na.rm=T), digits = 1))))}

if(nrow(sub2015) >0){
  plot(sub2015$date, sub2015$x, main=paste(filename,"_2015",sep=""), ylab = "Precipitation [mm/h]", xlab= "Time", type="h")
  legend("topleft", c("Range", as.character(round(range(sub2015$x, na.rm=T), digits = 1))))}

if(nrow(sub2016) >0){
  plot(sub2016$date, sub2016$x, main=paste(filename,"_2016",sep=""), ylab = "Precipitation [mm/h]", xlab= "Time", type="h")
  legend("topleft", c("Range", as.character(round(range(sub2016$x, na.rm=T), digits = 1))))}

dev.off()

}

dir(path="CuashiData")
final("Via2_Cuashi.csv")



### stuff ####

# Stuff not included; maybe useful later

# Find duplicates (when sort NA that indicate end start, sorted; overlaps disappear)
cont_collect_max <- aggregate(x = cont_collected$x, by = list(cont_collected$date), FUN=max, na.rm=TRUE)
cont_collect_max$x[!is.finite(cont_collect_max$x)] <- NA #restore NAs
cont_collect_min <- aggregate(x = cont_collected$x, by = list(cont_collected$date), FUN=min, na.rm=TRUE)
cont_collect_min$x[!is.finite(cont_collect_min$x)] <- NA #restore NAs

#check for inconsistencies
cont_collect_max$x_min=cont_collect_min$x
rm(cont_collect_min)
inconsistent = which(cont_collect_max$x != cont_collect_max$x_min)
if (any(inconsistent))
  warning(paste("duplicates found (e.g. ", paste(head(cont_collect_max[inconsistent,]), collapse=", ")))
cont_collect_max$x_min=NULL
cont_collected <- cont_collect_max
names(cont_collected)[1]="date"
rm(cont_collect_max)


#remove obsolete NAs added to end of files (Why should I remove NAs added to file start and end?)
obs_na=which(diff(cont_collected$date)<0)
cont_collected = cont_collected[-obs_na,]


start_date = as.POSIXct(strptime("2010-01-01 00:00", "%Y-%m-%d %H:%M", tz="etc/GMT-1"))  # nur WZ
end_date = as.POSIXct(strptime("2010-01-10 23:45", "%Y-%m-%d %H:%M", tz="etc/GMT-1"))  # nur WZ
seq(from = start_date , to=end_date, by='1 min')


#for import into CUAHSI, add missing fields and header##
#collect_file=paste0("CuashiData/", station_id_golm,"_merged.txt") # for CUASHI export

#write.table(file=collect_file, x=cbind("some","header"), row.names = FALSE, col.names = FALSE, quote = FALSE, sep="\t")
#write.table(file=collect_file, x=data.frame(station_id_golm, "rainfall", format(cont_collected$date,  
#           tz="Etc/GMT-1", format="%Y-%m-%d %H:%M:%S"), cont_collected$x),quote=FALSE, sep="\t", 
#           col.names = FALSE, row.names=FALSE, append=TRUE, na="-9999")


windows()
plot(cumsum(sub2014$VALOR.x), cumsum(sub2014$VALOR.y), pch=".", main= "2014", xlab=statname1, ylab=statname2)
abline(0,1, col="red")  
identify(cumsum(sub2014$VALOR.x), cumsum(sub2014$VALOR.y), labels = sub2014$POSIXDATE)

windows()
plot(cumsum(merge_data$x.x), cumsum(merge_data$x.y), pch=".",  main= "Station comparison using cumulative sum",sub="single files aggregated hourly,continous times series; merge (... all=FALSE);", xlab="Station 1", ylab="Station 2")
grid(5,5)
abline(0,1, col="orange") 
identify(cumsum(merge_data$x.x), cumsum(merge_data$x.y),labels = merge_data$date ) 


plot(merge_data$date, cumsum(merge_data$x.x), pch=".", ylim=c(0,max(c(cumsum(merge_data$x.x),cumsum(merge_data$x.y)))),main="Station comparison",sub="single files aggregated hourly, continous times series; merge (... all=FALSE)",xlab="Time", ylab="Cumulative Sum")
points(merge_data$date, cumsum(merge_data$x.y), pch=".", col="red")
points(merge_data$date, abs((cumsum(merge_data$x.x)-cumsum(merge_data$x.y))), pch=".", col="blue")
legend("topleft",c("Station 1","Station 2","|Station 1 - Station 2|"), pch=19, col=c("black","red","blue"))

data_1NA <- data_1[-which(is.na(data_1$x)),]
plot(data_1NA$date, cumsum(data_1NA$x), pch=".", main="Station 1",xlab="Time", ylab="Cumulative Sum")
abline(v=as.numeric(data_1$date[which(is.na(data_1$x))]), lwd=1, col='red')
identify(data_1NA$date, cumsum(data_1NA$x),labels = data_1NA$date ) 
