# turbidimeter data (Capella)
  
#merge files
  dfiles = dir (path = "campbell/", full.names = TRUE, pattern = "\\.")

  cont_all=NULL
  for (dfile in dfiles[-(1:6)])
  {
    paste0("reading ", dfile)
    if (grepl(dfile, pattern = "\\.xls")) #read excel
    {
      library(xlsx)
      cont = read.xlsx(file = dfile, sheetIndex = 1, startRow = 1, colIndex = 1:5, header = FALSE, stringsAsFactors=FALSE)
      summary(cont)
      names(cont)=paste0("V",1:5)
      cont$V1 = dfile #store origin
    } else #read csv
    {
      cont = read.table(dfile, header = FALSE, sep="", stringsAsFactors = FALSE, nrows=1)
      if (grepl(cont, pattern = ","))
        sep="," else
          if (grepl(cont, pattern = " "))
            sep=" " else
              sep="\t"
  
      cont = read.table(dfile, header = FALSE, sep=sep, stringsAsFactors = FALSE, strip.white = TRUE)
      for (i in ncol(cont):1)
        if (all(is.na(cont[,i])))
          cont[,i]=NULL
      cont$V1 = dfile #store origin
    }  
    cont_all = rbind(cont_all, cont)                  
  }
  summary(cont_all)  
  
  cont_all$date  = as.POSIXct(strptime(apply(FUN = paste, MARGIN = 1, cont_all[,2:3], collapse=" "), format = "%Y %j", tz="etc/GMT-1")) 
  cont_all$date  = cont_all$date  + (cont_all[,4] %/% 100) * 3600 + (cont_all[,4] %% 100) * 60
  
  head (cont_all)
  #treat different date format
  failed_conversion = is.na(cont_all$date)
  any(failed_conversion)
  sum(failed_conversion)
  cont_all[failed_conversion,]
  
  #sort
  cont_all = cont_all[sort.int(cont_all$date, index.return = TRUE)$ix,]
  ddates = duplicated(cont_all[,-1]) #find consistent duplicates
  
  cont_all = cont_all[!ddates,] #remove duplicates
  
  nrow(cont_all)
  ddates = duplicated(cont_all$date) #find inconsistent duplicates
  sum(ddates)
  if (any(ddates))
  {
    ddates_r = duplicated(cont_all$date, fromLast=TRUE)
    plot(cont_all$V5[ddates], cont_all$V5[ddates_r])  
    a=identify(cont_all$V5[ddates], cont_all$V5[ddates_r], label=cont_all$date[ddates])
    cont_all[ cont_all$date %in% cont_all$date[ddates][a],]
    table(cont_all$V1[ddates_r] )
    
    
    # #remove the duplicates caused by x201001-201406.dat (presumably already postprocessed)
    # cont_all = filter(cont_all, !((cont_all$V1 == "campbell/x201001-201406.dat") & (ddates | ddates_r) ))
    # 
    # ddates = duplicated(cont_all$date) #find inconsistent duplicates
    # ddates_r = duplicated(cont_all$date, fromLast=TRUE)        
    # plot(cont_all$V5[ddates], cont_all$V5[ddates_r])  #if the remaining duplicates are just caused by different numeric precision, discard duplicates
    library(dplyr)
    cont_all = filter(cont_all, !ddates_r ) #remove duplicates
  }
   
  cont_all$V1 = NULL
  
  
  
  #check for faulty data
  faulty_values = is.na(as.numeric(cont_all$V5))
  any(faulty_values)
  sum(faulty_values)
  cont_all = cont_all[!faulty_values,]
  
  names(cont_all) = c("year","doy","time","ssc", "date")
  cont_all$year = NULL
  cont_all$doy = NULL
  cont_all$time = NULL
  summary(cont_all)
  
  cont_all = cont_all[cont_all$ssc != -6999,] #remove NA-values
  
  save(file="Capella_turb_merged.RData", list="cont_all")
  load(file="Capella_turb_merged.RData")
  
  #phase A (Endress + Hauser, postprocessed by Jos) - contains 3 calibration phases: before 30.11.11 and another break  between 22.3.12 and 4.6.12. Waiting for Jos's response
  cont_all = cont_all %>% 
    #filter(date < as.POSIXct(ISOdate(year = 2013, month = 11, day=10, hour=20, min = 30, tz="etc/GMT-1")))
    filter(date < as.POSIXct(ISOdate(year = 2015, month = 11, day=6,  hour=10, min = 45, tz="etc/GMT-1")))

  #second phase (Endress + Hauser, raw): no samples contained, needs to be connected to first phase
  # library(dplyr)
  # cont_all <- cont_all %>%
  #            filter(date >= as.POSIXct(ISOdate(year = 2013, month = 11, day=10, hour=20, min = 30, tz="etc/GMT-1"))) %>%
  #            filter(date <  as.POSIXct(ISOdate(year = 2015, month = 11, day=6,  hour=10, min = 45, tz="etc/GMT-1")))
  #   
  #third phase (WTW ViSolid)
  #cont_all = cont_all[cont_all$date >= as.POSIXct(ISOdate(year = 2015, month = 11, day=6, hour=10, min = 45, tz="etc/GMT-1")),]
  
  hist(cont_all$ssc)
  summary(cont_all)
  
  plot(cont_all$date, cont_all$ssc, type="p", pch=".")

#load lab results
  library(xlsx)
  samples = read.xlsx(file = "../ISCO_laboranalysen_201109-201604_5.xlsx", sheetName = "cap", startRow = 2, header = TRUE, stringsAsFactors=FALSE)  
  summary(samples)
  #samples$date = as.POSIXct(strptime(paste(samples$Date.ISCO.Recording, format(samples$time,"%H:%M")), format = "%Y-%m-%d %H:%M", tz="etc/GMT-1"), tz="etc/GMT-1")
  samples$date = as.POSIXct(strptime(samples$time.corrected..GMT.1., format = "%Y-%m-%d %H:%M", tz="etc/GMT-1"), tz="etc/GMT-1")

  faulty_values = is.na(as.numeric(samples$Concentration..g.L.))
  print(paste0(sum(faulty_values), " non-finite SCCs found"))
  head(samples[faulty_values,])
  samples$Concentration..g.L. = as.numeric(samples$Concentration..g.L.)

  samples = samples[!is.na(samples$Concentration..g.L.),] #remove nodata values in SSC

  neg_values = samples$Concentration..g.L. < 0
  print(paste0(sum(neg_values), " negative SCCs found"))
  
  samples = samples[samples$Concentration..g.L. > 0,] #remove negative SSC
  
  summary(samples)
  
  which(is.na(samples$date))
  
#plot both  
  subs = seq(from=1, to=nrow(cont_all), by=5) #take only every nth point otherwise playwith is too slow
  windows()
  library(playwith)
  playwith(
    plot(cont_all$date[subs], abs(cont_all$ssc[subs]), type="p", pch=".")       
  )
  points(samples$date, samples$Concentration..g.L., col="red") #copy for documentation
  
  
  
#relate turbidity and ISCO samples  
  samples$Concentration..g.L.[samples$Concentration..g.L. == -1000] = NA #remove nocont_all values
  covered = (samples$date > min(cont_all$date)) & (samples$date < max(cont_all$date))
  samples = samples [covered,] #exclude values outside turbidimeter series

  find_closest=function(series, target) #find closest match of target in series
  {return(which.min(abs(series-target))[1])}

  #try to find corresponding time
  meas_times = sapply(X=samples$date, FUN = find_closest, series=cont_all$date) #try to find corresponding time

  #difference in time between sample and turbidimeter record
  temp_difference = difftime(samples$date, cont_all$date[meas_times], units = "min")
  no_correspondence = abs(as.numeric(temp_difference)) > 15 #find records without directly corresponding
  sum(no_correspondence, na.rm=TRUE) #number of SSC-samples omitted becuase of no correspondence
  meas_times[no_correspondence] = NA
  
  samples_m = cbind(samples, turb = cont_all[meas_times, "ssc"])
  samples_m = samples_m[!is.na(samples_m$turb),] #discard NAs

  nrow(samples_m)     
  
  color = 60*(as.numeric(samples_m$date) - as.numeric(min(samples_m$date))) / diff(range(as.numeric(samples_m$date))) + 1
  playwith(
  plot( samples_m$turb, samples_m$Concentration..g.L., col=color)
)
   #      , xlim=c(0,200))
  cor(samples_m$Concentration..g.L., samples_m$turb )
  
  # #plot selected examples
  # windows()
  # start = as.POSIXct(strptime("2013-03-16 02:51:42", format = "%Y-%m-%d %H:%M:%S"))
  # stop   =as.POSIXct(strptime("2013-04-02 11:41:54", format = "%Y-%m-%d %H:%M:%S"))
  # 
  # start = as.POSIXct(strptime("2014-03-28 17:00:04", format = "%Y-%m-%d %H:%M:%S"))
  # stop   =as.POSIXct(strptime("2014-05-21 21:10:59", format = "%Y-%m-%d %H:%M:%S"))
  # 
  # valid_cont_all  = cont_all$date >= start & cont_all$date <= stop
  # plot(cont_all$date[valid_cont_all], cont_all$x[valid_cont_all], type="p", pch=".", ylim=c(0,100))       
  # points(samples_m$date, samples_m$Concentration..g.L., ylim=c(0, 50), col=color)
  
  
  plot_turb <- function(x, y, ...) { #subs, ssc_corr=NULL,
    #plot(cont_all$date[subs], cont_all$ssc[subs], type="p", pch=".", ylim=ylim, xlim=xlim)
    #plot(cont_all$date[subs], cont_all$ssc[subs], type="p", pch=".", ...)
    plot(x, y, type="p", pch=".", axes=FALSE, ...) 

   points(samples_m$date, samples_m$Concentration..g.L., col=color)
  
   if (exists("turb0"))
     points(cont_all$date[subs], turb0[subs], col="green", pch=".")
   
   if (exists("breakpoints"))
     points(breakpoints$date, breakpoints$ssc, col="black", pch="x")
   
   if (exists("start_stop"))
   {  
     abline(v=x[start_stop_m[,1]], col="green")
     abline(v=x[start_stop_m[,2]], col="red")
   }  
   
   
   
    if (!exists("xlim") || is.null(xlim)) xlim=range(x, na.rm=TRUE)
    if ("POSIXt" %in% attr(x,"class") ) #if x-data is a time
    {
      org_tz=attr(x,"tzone")  #get timezone of original x-data
      if (is.numeric(xlim[1]))  #sometimes during zooming, xlim is used in numeric form instead of POSIX-format
        xlim =  as.POSIXct(xlim, origin="1970-01-01 00:00.00 UTC")
      xlim = as.POSIXct(as.POSIXlt(xlim, tz=org_tz))
    }
    ticks=pretty( xlim)
    a=axis(side=1, at=ticks, labels=ticks)
    a=axis(side=2)

    
     }
  
  # windows()
  # plot_turb()
  # 
  
  
  try(load(file="turb_phases.RData"), silent = TRUE )
  if (exists("start_stop"))
    start_stop_m = matrix(start_stop, ncol=2, byrow = TRUE) else
      start_stop = start_stop_m = NULL
  myIds=NULL
  
#manually determine start and stop of usable phases, export as "myIds"
  library(playwith)
  subs = seq(from=1, to=nrow(cont_all), by=10)
  if (exists("valid_cont_all"))
    col = 1+as.numeric(!valid_cont_all[subs])  else
    col=1
  playwith(plot_turb(x=cont_all$date[subs], y=cont_all$ssc[subs], col=col))
  
  #start_stop = which(diff(valid_cont_all)!=0)
  start_stop = sort(unique(c(start_stop, myIds)))
  start_stop_m = matrix(start_stop, ncol=2, byrow = TRUE)
  valid_cont_all = rep(FALSE, nrow(cont_all))
  for (i in 1:NROW(start_stop_m))
  {
    begin = cont_all$date[subs][start_stop_m[i,1]]
    end   = cont_all$date[subs][start_stop_m[i,2]]
    valid_cont_all [cont_all$date >= begin & cont_all$date <= end] = TRUE
  }  
  save(file="turb_phases.RData", list="start_stop")  

#manually define zero level by clicking along graph, export as "myIds"  
  #subs = seq(from=1, to=sum(valid_cont_all), by=10) #take only every 10th point otherwise playwith is too slow
  breakpoints = NULL
  myIds=NULL
  load(file="breakpoints.RData")  
  print("manually define zero level by clicking along graph now, export labels as 'myIds'   ")
  playwith(plot_turb(x=cont_all$date[subs], y=cont_all$ssc[subs], col=col))
  
  
  # #phase a)
  # cont_all$ssc[which(cont_all$ssc < -1)]=NA #discard the few erroneous measurement
  # breakpoints = data.frame(date=cont_all$date[c(1, nrow(cont_all))], ssc=min(cont_all$ssc[valid_cont_all], na.rm=TRUE))  #take it all
  #  
   
  breakpoints = unique(rbind(breakpoints, cont_all[,][subs,][myIds,c("ssc","date")]))
  breakpoints = breakpoints[sort.int(breakpoints$date, index.return = TRUE)$ix,]
  
  save(file="breakpoints.RData", list = c("valid_cont_all", "breakpoints")) 
  load(file="breakpoints.RData")
  
  t0_fun = approxfun(x = breakpoints$date, y=breakpoints$ssc, rule = 2)
  
  #save(file="all", list = ls()) 
  
  turb0 = t0_fun(cont_all$date)  #compute zero-level of turbidity
  playwith(plot_turb(x=cont_all$date[subs], y=cont_all$ssc[subs], col=col))
  cont_all$turb0 = turb0
  rm(turb0)
  tsign = sign(median(cont_all$ssc[valid_cont_all], na.rm=TRUE)+1) #capture case of inverted turbidimeter record
  cont_all$turb_corr = tsign*pmax(0, tsign*(cont_all$ssc - cont_all$turb0))  #zero-corrected turbidity
  
#plot zero-corrected and validity-selected data
  rm(start_stop)
  rm (breakpoints)
  playwith(plot_turb(x=cont_all$date[valid_cont_all][subs], y=cont_all$turb_corr[valid_cont_all][subs]))

  
  #check relationship with corrected zero level
    samples_m$turb_corr = tsign*pmax(0, tsign*(samples_m$turb - t0_fun(samples_m$date)))
    #samples_m = samples_m[-3,]  #for last phase, this sample seems problematic
    samples_m$turb_corr[!valid_cont_all[na.omit(meas_times)]] = NA
    windows()
    plot(samples_m$turb_corr, samples_m$Concentration..g.L., col=color)
    cor(samples_m$Concentration..g.L., samples_m$turb_corr, use = "complete.obs")
   identify(samples_m$turb_corr, samples_m$Concentration..g.L., labels = samples_m$date)

   
#establish regression
   #cal_turb = lm(Concentration..g.L. ~ I(turb_corr*turb_corr)+turb_corr -1, data=samples_m)    
   cal_turb = lm(Concentration..g.L. ~ turb_corr, data=samples_m)    
   summary(cal_turb)
   
   # cal_turb = lm(log(Concentration..g.L.) ~ log(turb_corr), data=samples_m, subset = samples_m$Concentration..g.L.*samples_m$turb_corr !=0)    
   # summary(cal_turb) #worse than polynom
    
   turb2plot = seq(from=0, to=max(samples_m$turb_corr, na.rm=TRUE), length.out=30)
   lines(turb2plot, predict(cal_turb, newdata = data.frame(turb_corr=turb2plot)))  
   #lines(turb2plot, exp(predict(cal_turb, newdata = data.frame(turb_corr=turb2plot))))  
    #export this for docu
   
   
   
       
#export for CUAHSI      
      source("f_write4cuahsi.R")
      
      #write turbidity
      cont_all$DataValue = cont_all$ssc
      cont_all$Offset    = cont_all$turb0
      cont_all$QualityControlLevelCode = NA
      cont_all$QualityControlLevelCode[valid_cont_all] =  5 #tag invalid data periods
      
      write4cuahsi(cdata = cont_all, dest_file = paste0("../CuahsiData/cap_turb.txt"), 
                   varcode = "turbidity15", sitecode <- "Capella", sourcec = "SESAMII_hydro", methodc="turbidimeter")
      
      #write sediment concentration
      cont_all$DataValue = predict(cal_turb, newdata = data.frame(turb_corr=cont_all$turb_corr))
      cont_all$Offset    = NULL
      write4cuahsi(cdata = cont_all, dest_file = paste0("../CuahsiData/cap_ssc.txt"), 
                   varcode = "ssc", sitecode <- "Capella", sourcec = "SESAMII_hydro", methodc="turbidimeter")
      
      playwith(plot_turb(x=cont_all$date[valid_cont_all][subs], y=cont_all$DataValue[valid_cont_all][subs]))
      
      library(dplyr)
      summary(filter (cont_all, valid_cont_all == TRUE) )
      
      